!-------------------------------------------------------------------------------
! Atmospheric Dispersion And Parallel Transport (ADAPT) program to evaluate
! various OMP parallel statements in the context of atmospheric dispersion 
! calculations. Program elements including reading ARL packed meteorological 
! data files, converting meteorological data to computational units, 
! computing particle advection and dispersion, and the accumulation of
! particle mass on a concentration grid. The current version only supports the
! use of WRF meteorological data for transport and TKE fields for mixing.
! This computer code is distributed under the GNU LGPL license, 
! https://www.gnu.org/licenses/lgpl-3.0.en.html.
! Copyright: roland.draxler@meteozone.com
!-------------------------------------------------------------------------------
! Compilation requires the conformal mapping conversion library:
! MAP="libcmapf.a"   
! Available from two different sources:
! https://www.arl.noaa.gov/wp_arl/wp-content/uploads/utilities/cmap/
!         cmapf.v1_0.tar.gz
! https://www.iamg.org/documents/oldftp/VOL23/
!         v23-1-5.tar.Z
!-------------------------------------------------------------------------------
! Use the following statements to compile the code with gfortran:
! OPT="-ffree-form -fconvert=big-endian -frecord-marker=4"
! gfortran -oadapt -fopenmp ${OPT} ${MAP} adapt.f 
!-------------------------------------------------------------------------------
! Simulations default to the number of cores, unless ...
! export OMP_NUM_THREADS={X
!-------------------------------------------------------------------------------
! Last Revised: 13 Dec 2019 - restructured for simple vertical grid
!               04 Aug 2020 - use RAN1 to assign random number to particle array
!               08 Aug 2020 - revised close coupling; concentration common block
!               13 Aug 2020 - option for finer concentratiom (cgratio) grid
!                             in conjunction with output in DATEM format; enable
!                             the dat??? routines and disable con??? routines
!               30 Oct 2020 - added trajectory feature for test purposes
!               11 Nov 2020 - added simultaneous multiple time scales
!               12 Nov 2020 - moved configuring meteo data to subroutine
!               19 Nov 2020 - added WRF time averaged mass velocities
!               21 Nov 2020 - namelist for simulation configuration
!               30 Dec 2020 - input vertical coordinate system options
!               01 Jan 2021 - increased KZMAX to account for HRRR
!               11 Jan 2021 - estimate TKE when missing
!               15 Jan 2021 - added back transport option; pressure level data
!               30 Jan 2021 - added SVG output option for concentration field
!               11 Feb 2021 - error check when reading SETUP.CFG
!-------------------------------------------------------------------------------

MODULE metval
! meteorological grid spacing
  REAL,    ALLOCATABLE :: xd (:,:)     ! u grid distance (m/grid)
  REAL,    ALLOCATABLE :: yd (:,:)     ! v grid distance (m/grid)
  REAL,    ALLOCATABLE :: zd (:)       ! z grid distance between K and K-1 (m)
  REAL,    ALLOCATABLE :: zt (:)       ! total grid distance from k=1 (m) 
  REAL,    ALLOCATABLE :: zk (:)       ! fractional index by height in meters
  REAL,    ALLOCATABLE :: glat (:,:)   ! grid point in earth units (latitude)
  REAL,    ALLOCATABLE :: glon (:,:)   ! grid point in earth units (longitude)

! three dimensional meteorological variables
  REAL,    ALLOCATABLE :: u  (:,:,:,:) ! u wind (m/sec)
  REAL,    ALLOCATABLE :: v  (:,:,:,:) ! v wind (m/sec)
  REAL,    ALLOCATABLE :: w  (:,:,:,:) ! z wind (m/sec)

  REAL,    ALLOCATABLE :: p  (:,:,:,:) ! pressure
  REAL,    ALLOCATABLE :: t  (:,:,:,:) ! temperature
  REAL,    ALLOCATABLE :: q  (:,:,:,:) ! specific humidity
  REAL,    ALLOCATABLE :: e  (:,:,:,:) ! turbulent kinetic energy (J/kg)

! turbulence variables (square root of variance)
  REAL,    ALLOCATABLE :: uu (:,:,:,:) ! u turbulence (m/s)
  REAL,    ALLOCATABLE :: vv (:,:,:,:) ! v turbulence (m/s)
  REAL,    ALLOCATABLE :: ww (:,:,:,:) ! z turbulence (m/s)
  REAL,    ALLOCATABLE :: hh (:,:,:)   ! integrated hypsometric (m)

! surface variables
  REAL,    ALLOCATABLE :: zi (:,:,:)   ! pbl height
  REAL,    ALLOCATABLE :: rt (:,:,:)   ! precipitation
  REAL,    ALLOCATABLE :: uf (:,:,:)   ! friction velocity
  REAL,    ALLOCATABLE :: hf (:,:,:)   ! heat flux
  REAL,    ALLOCATABLE :: ds (:,:,:)   ! downward shortwave flux
  REAL,    ALLOCATABLE :: sp (:,:,:)   ! mean sea level pressure      

! mass flux variables for time averaged WRF
  REAL,    ALLOCATABLE :: msfu (:,:)   ! U=u*msfu/muu0
  REAL,    ALLOCATABLE :: msfv (:,:)   ! V=v*msfv/muv0
  REAL,    ALLOCATABLE :: msft (:,:)   ! W=w*msft/100.0
  REAL,    ALLOCATABLE :: muu0 (:,:)  
  REAL,    ALLOCATABLE :: muv0 (:,:) 

  SAVE
END MODULE metval

MODULE ranval
! random number generator variables
  INTEGER              :: iseed = -320
  INTEGER,   PARAMETER :: maxran=2000000 
  REAL,    ALLOCATABLE :: rannum(:)

! Lagrangian Time Scale bins for vertical and horizontal (hscale) in seconds
! where vscale(s|u) for stable and unstable
  INTEGER  :: nbinv = 1  ! number of time scale bins
  REAL     :: vscales(1) = (/5.0/)  
  REAL     :: vscaleu(1) = (/200.0/) 
!#REAL     :: vscales(1) = (/5.0/)
!#REAL     :: vscaleu(1) = (/5.0/) 

  INTEGER  :: nbinh = 1  ! number of time scale bins
  REAL     :: hscale(1)  = (/10800.0/)  

! autocorrelation for various timescales
  REAL,    ALLOCATABLE :: rautos(:,:)
  REAL,    ALLOCATABLE :: rautou(:,:)
  REAL,    ALLOCATABLE :: rautoh(:,:)

! product of the autocorrelation
  REAL,    ALLOCATABLE :: pautos(:,:)
  REAL,    ALLOCATABLE :: pautou(:,:)
  REAL,    ALLOCATABLE :: pautoh(:,:)

  SAVE
END MODULE ranval

MODULE datval
! number of DATEM records and header INFO records
  INTEGER        :: NREC
  CHARACTER(80)  :: INFO(2)

! sample start-stop and duration time variables
  INTEGER, ALLOCATABLE :: JYR(:),JMO(:),JDA(:),JHR(:),JMN(:)
  INTEGER, ALLOCATABLE :: KYR(:),KMO(:),KDA(:),KHR(:),KMN(:)
  INTEGER, ALLOCATABLE :: LHR(:),LMN(:),KSTA(:)

! sample identification variables
  REAL,    ALLOCATABLE :: SLAT(:),SLON(:),XPOS(:),YPOS(:)
! sample concentration and counter variables
  REAL,    ALLOCATABLE :: CALC(:),CNTR(:)

  SAVE
END MODULE datval

!-------------------------------------------------------------------------------

PROGRAM ADAPT    

  USE metval
  USE ranval

  IMPLICIT NONE

  INTEGER  :: jet             ! current time (sec)
  INTEGER  :: dt              ! current time step (sec) 
  INTEGER  :: dte             ! maximum time step for emissions (sec)
  INTEGER  :: dta             ! maximum time step for advection (sec)
  INTEGER  :: delm            ! time interval between meteorology (sec)
  INTEGER  :: dinp            ! internal time for meteorology input (sec)
  REAL     :: tker            ! ratio of vertical to horizontal turbulence
  REAL     :: tratiov         ! ratio for vertical time step stability
  REAL     :: tratioh         ! ratio for horizontal time step stability
  REAL     :: tkemax          ! max value for turbulent kinetic energy (J/kg)
  REAL     :: tkemin          ! min value for turbulent kinetic energy (J/kg)
  REAL     :: pblmin          ! minimum value for the PBL depth (m)
  REAL     :: zf              ! fractional vertical index

  INTEGER  :: kunit=10        ! meteorological data unit number
  INTEGER  :: junit=20        ! concentration output file unit number
  INTEGER  :: munit=30        ! trajectory output file unit number
  INTEGER  :: nunit=40        ! html output in SVG format
  INTEGER  :: iunit=50        ! map file for SVG plotting
  INTEGER  :: k1=0            ! time index for last time's data
  INTEGER  :: k2=1            ! time index for next time's data
  INTEGER  :: nxp             ! number of horizontal W->E grid points
  INTEGER  :: nyp             ! number of horizontal S->N grid points
  INTEGER  :: nzp             ! number of vertical levels (including sfc)
  INTEGER  :: kzmax           ! model top in meters, must exceed data top

  LOGICAL  :: emit            ! internal flag: particle emissions on when true
  LOGICAL  :: tavg            ! internal flag: time averaged meteo data 
  LOGICAL  :: traj            ! trajectory emits only once
  LOGICAL  :: tked            ! tke data in meteorology    
  LOGICAL  :: zgrid           ! when false configure the vertical grid only once
  LOGICAL  :: back            ! when true integration is backward in time

! initial particle position
  REAL     :: x0,y0,z0        ! release position in grid units
  REAL     :: olat,olon,olvl  ! release position in earth units 
  INTEGER  :: crun            ! run duration in hours
  INTEGER  :: emdur           ! emission duration in minutes
  INTEGER  :: pemit           ! relative minutes to start emissions

! trajectory variables
  REAL     :: tlat,tlon,tzht  ! position
  REAL     :: tmetv           ! trajectory meteorological variable

! particle array (PA) element description equated to variables in ADVIEC
        REAL, ALLOCATABLE :: pa(:,:)
! pa=1  REAL, ALLOCATABLE :: pm(:)  ! particle mass (user units)    
! pa=2  REAL, ALLOCATABLE :: xp(:)  ! x-particle position (grid-pt)
! pa=3  REAL, ALLOCATABLE :: yp(:)  ! y-particle position (grid-pt)
! pa=4  REAL, ALLOCATABLE :: zp(:)  ! z-particle position (grid-pt)
! pa=5  REAL, ALLOCATABLE :: up(:)  ! u-particle turbulence (m/s)
! pa=6  REAL, ALLOCATABLE :: vp(:)  ! v-particle turbulence (m/s)
! pa=7  REAL, ALLOCATABLE :: wp(:)  ! w-particle turbulence (m/s)
! pa=8  REAL, ALLOCATABLE :: wr(:)  ! w-turbulence ratio (wp/sig(wp))
! pa=9                              ! sequential particle number index
! pa=10                             ! random array pointer index

  REAL, ALLOCATABLE :: cs (:,:,:)   ! concentration summation
  REAL, ALLOCATABLE :: ct (:,:,:)   ! concentration temporary

! diagnostic variables 
  LOGICAL           :: diag         ! output diagnostics
  LOGICAL           :: ftest        ! file exist test
  REAL              :: tmass        ! total mass diagnostic
  REAL, ALLOCATABLE :: mass(:)      ! mass by height 

! particle and emission variables
  REAL           :: totrel          ! total mass released 
  REAL           :: pmassv          ! particle value of total mass
  REAL           :: up0,vp0,wp0,wr0 ! intialization for particle turbulence
  REAL           :: volume          ! sampling cell volume (m3)
  REAL           :: parden          ! particle density (#/m3)
  REAL           :: parage          ! particle age (s)
  INTEGER        :: numpar          ! current active number of particles 
  INTEGER        :: maxpar          ! maximum number for array allocation
  INTEGER        :: paremt          ! particle release rate per (delm/dt)
  INTEGER        :: numemp          ! number of emission periods

! pollutant and concentration grid
  LOGICAL        :: datem           ! output in DATEM format
  LOGICAL        :: html            ! output in HTML format as SVG
  LOGICAL        :: hybin           ! output in HYSPLIT binary format
  LOGICAL        :: thin            ! no headers in HTML output
  REAL           :: cfactor         ! concentration conversion factor
  CHARACTER(256) :: datfile         ! input file when datem=T
  CHARACTER(256) :: confile         ! HYSPLIT binary concentration output
  CHARACTER(256) :: stnfile         ! concentration output in DATEM format
  CHARACTER(256) :: trjfile         ! trajectory endpoints in HYSPLIT format
  CHARACTER(256) :: svgfile         ! concentration SVG file when html=T   
  CHARACTER(256) :: mapfile         ! background map file for SVG plots 
  CHARACTER(4)   :: polid           ! pollutant identification
  INTEGER        :: conlvl          ! concentration output level (m AGL)
  REAL           :: clat            ! lower left corner concengration grid
  REAL           :: clon            ! lower left corner concentration grid
  REAL           :: dlat            ! grid spacing in degrees latitude
  REAL           :: dlon            ! grid spacing in degrees longitude
  INTEGER        :: nlon            ! number of longitude grid points
  INTEGER        :: nlat            ! number of latitude grid points
  INTEGER        :: nxc,nyc         ! would equal nxp,nyp when cgratio=1
  INTEGER        :: cgratio         ! numb of conc grid cells to WRF cells
  INTEGER        :: cstart          ! concentration output start time (sec)
  INTEGER        :: cinter          ! concentration output interval (sec)

! meteorological data file structure
  INTEGER        :: ntp             ! number of time periods
  INTEGER        :: krec,jrec       ! input data record number
  INTEGER        :: numndx          ! number of index records
  INTEGER        :: numper          ! number of records per time period
  INTEGER        :: nvsfc,nvlvl     ! number variables at surface and aloft

! meteorological input file grid variables
  CHARACTER(4)   :: kvar
  CHARACTER(50)  :: label 
  CHARACTER(108) :: header
  CHARACTER(256) :: metfile(5)       ! meteorological data file names
  INTEGER        :: lrec             ! length of meteorological record
  INTEGER        :: mfile = 1        ! meteo file pointer for metfile 
  REAL           :: ptop             ! top pressure for hydrostatic
  REAL           :: sg(100)          ! hydrostatic sigma levels

! grid offset of WRF flux from mass points
  REAL           :: sux,suy,svx,svy

! conformal map projection variables 
  REAL           :: gbase(9)   

! integration loop indicies
  INTEGER        :: nb,ns,np  
  INTEGER        :: nt,nz   
  INTEGER        :: ii,jj,kk

! error return codes
  INTEGER        :: kret   
  CHARACTER(256) :: ecode  

! date-time in six element array ->  (yy,mm,dd,hh,mm,ff)
  INTEGER        :: dateprv(6) ! previous time meteorology
  INTEGER        :: dateinp(6) ! input time meteorology
  INTEGER        :: datecal(6) ! computational start time
  INTEGER        :: daterel(6) ! particle release start time
  INTEGER        :: datebgn(6) ! output begin time
  INTEGER        :: dateend(6) ! output ending time

! variables for CPU internal time and clock functions
  REAL           :: SS,HOURS
  INTEGER(KIND=8):: DSEC,RSEC,CSEC,BSEC,JULSEC
  INTEGER        :: DELTA
  INTEGER        :: CENTURY
  INTEGER        :: YR,MO,DA,HR,MN
  INTEGER        :: DATE_TIME(8)
  CHARACTER(12)  :: REAL_CLOCK(3)

! define OMP thread functions and variables
! note that the pre-processor definition OMP conflicts with the
! gfortran compiler -fopenmp option                

  INTEGER        :: kmax, OMP_GET_MAX_THREADS
  INTEGER        :: kthr, OMP_GET_THREAD_NUM
  INTEGER        :: kcpu, OMP_GET_NUM_PROCS
! INTEGER        :: kmax
! INTEGER        :: kthr
! INTEGER        :: kcpu

! random number generator functions
  REAL           :: rvalue
  REAL           :: r4_normal_01, r4_uniform_01

  INTEGER        :: nrand
  LOGICAL        :: fixed

!-------------------------------------------------------------------------------

! interface statements 
  INCLUDE 'adapt.inc'        

!-------------------------------------------------------------------------------

  COMMON /stblty/ tratiov,tratioh,tkemin,tkemax,pblmin
  COMMON /gridpt/ nxp,nyp,nzp,kzmax,cgratio
  COMMON /offset/ sux,suy,svx,svy 
  COMMON /conval/ polid,conlvl,clat,clon,dlat,dlon,nlat,nlon

  NAMELIST /SETUP/ back,traj,zgrid,fixed,datem,html,thin,hybin,diag,   &
            datfile,metfile,confile,stnfile,trjfile,svgfile,mapfile,   &
            polid,totrel,cfactor,olat,olon,olvl,datecal,crun,          &
            daterel,emdur,maxpar,cgratio,conlvl,cstart,cinter,         &
            dte,dta,tratioh,tratiov,tker

!--------------------------------
  CALL DATE_AND_TIME(REAL_CLOCK(1),REAL_CLOCK(2),REAL_CLOCK(3),DATE_TIME)
  READ(REAL_CLOCK(1),'(I4,2I2)')YR,MO,DA   
  READ(REAL_CLOCK(2),'(2I2,F6.3)')HR,MN,SS
  DSEC=JULSEC(YR,MO,DA,HR,MN,INT(SS))
!--------------------------------

  back=.false.    ! when true integration is backward in time
  traj=.false.    ! when true compute a single particle trajectory
  zgrid=.false.   ! when false configure the vertical grid only once
  fixed=.false.   ! start with fixed index or random index
  datem=.false.   ! output in DATEM format
  html=.false.    ! concentration output in html format
  thin=.false.    ! when true strips headers from html output file
  hybin=.true.    ! concentration output in HYSPLIT binary format
  diag=.false.    ! output diagnostics

! measured datem input file -> output to stnfile if datem=true 
  datfile='measured_datem.txt'
! calculated concentration output file in DATEM format
  stnfile='calculated_datem.txt'
! concentration binary output file in HYSPLIT format
  confile='concout_adapt.bin' 
! trajectory endpoints output file in HYSPLIT format
  trjfile='endpts_adapt.txt'
! concentration file {base}{00.html} name for output as SVG
  svgfile='svgplot'
! graphics map background
  mapfile='map_standard.txt'

! first meteorological data file
  metfile(1)='/Tutorial/captex/captex2_wrf27uw.bin'
  metfile(2:5)=''

  polid='PMCH'    ! pollutant identification field
  totrel=201000.0 ! total mass released 
  maxpar=50000    ! maximum number for array allocation
  cfactor=1.0     ! units conversion for output

  olat=39.90  
  olon=-84.22  
  olvl=10.0  

! the computational start time defaults to current time so that the
! field is defined in the event it is missing in SETUP.CFG
! minutes (5) and forecast hour (6) always default to zero
  datecal(1)=1983          ! date_time(1)-2000
  datecal(2)=9             ! date_time(2)
  datecal(3)=25            ! date_time(3)
  datecal(4)=17            ! date_time(5)
  datecal(5)=0            
  datecal(6)=0

  crun=64         ! run length in hours
  daterel=datecal ! particle release start time
  emdur=180       ! emission duration in minutes

  cgratio=1       ! number of concentration cells per WRF cell
  conlvl=100      ! vertical average of WRF levels for output
  cstart=60       ! concentration output start from datecal (min)
  cinter=180      ! concentration output interval (min)

  dte=300         ! maximum emission time step (sec)
  dta=1200        ! maximum advection time step (sec)
  tratioh=0.50    ! ratio for horizontal time step stability
  tratiov=0.10    ! ratio for vertical time step stability
  tker=0.18       ! ratio of vertical to horizontal turbulence

!--------------------------------
! internally set variables; those above set in NAMELIST

  tkemax=5.0      ! maximum value for turbulent kinetic energy
  tkemin=0.05     ! minimum value for turbulent kinetic energy
  pblmin=50.0     ! minimum value for the PBL depth  
  kzmax=34000     ! maximum height (m-AGL) of vertical grid

  SUX=0.5         ! stagger of the u,v points relative to mass points
  SUY=0.0
  SVX=0.0
  SVY=0.5

!--------------------------------
! read the namelist file or create if it does not exist 

  INQUIRE(FILE='SETUP.CFG',EXIST=FTEST)
  IF(FTEST)THEN
     OPEN(kunit,FILE='SETUP.CFG')
     READ(kunit,SETUP,IOSTAT=kret)
     IF(kret.EQ.0)THEN
        CLOSE(kunit)
        WRITE(*,*)'Namelist read to configure calculation' 
     ELSE
        WRITE(*,*)'Error reading one of the namelist variables'
        STOP
     END IF
  ELSE
     OPEN(kunit,FILE='SETUP.CFG')
     WRITE(kunit,SETUP)
     CLOSE(kunit)
     WRITE(*,*)'Edit SETUP.CFG to configure calculation' 
     STOP
  END IF

  IF(traj)THEN
!    for trajectories over-ride the emission and number
     totrel=1.0
     maxpar=1
  END IF

! adjust datecal and daterel fields for the proper century
! assuming that a YY entry is 20YY while a four digit 
! entry of YYYY could be any century

  IF(datecal(1).LT.100) datecal(1)=2000+datecal(1)
  IF(daterel(1).LT.100) daterel(1)=2000+daterel(1)

  cstart=cstart*60 ! concentration output start from datecal (sec)
  cinter=cinter*60 ! concentration output interval (sec)

!--------------------------------
! open the meteorological data file and set related varables

  CALL METSET (kunit,metfile,traj,tavg,tked,nxp,nyp,nzp,nvsfc,nvlvl,  &
               numper,numndx,delm,ptop,sg)

  LREC=50+nxp*nyp
  IF(diag)THEN
     WRITE(*,*) 'Tavg: ',tavg
     WRITE(*,*) 'Grid: ',nxp,nyp,nzp
     WRITE(*,*) 'Nvar: ',nvsfc,nvlvl
     WRITE(*,*) 'Nrec: ',numper
     WRITE(*,*) 'Nndx: ',numndx
     WRITE(*,*) 'DelM: ',delm
     WRITE(*,*) 'Lrec: ',lrec
     WRITE(*,*) 'Ptop: ',ptop
     IF(ptop.GE.0.0) THEN
        WRITE(*,'(A)')'Level   Sigma'
        DO kk=1,nzp
           WRITE(*,'(I5,2X,F6.4)')kk,sg(kk)
        END DO
     END IF
     WRITE(*,*)
  END IF

!--------------------------------
! compilation number of threads ..... or set the environment variable
! OMP PARALLEL NUM_THREADS(8)         export OMP_NUM_THREADS=8 

  kcpu=OMP_GET_NUM_PROCS()
  kmax=OMP_GET_MAX_THREADS()

! need to define if not using OMP library 
! kcpu=1
! kmax=1

  IF(kcpu.GT.kmax)THEN
     WRITE(*,*)'Total number of processors available: ',kcpu
     WRITE(*,*)'Number of threads allocated this run: ',kmax
     WRITE(*,*)'Export OMP_NUM_THREADS={x} to change! '
  ELSE
     WRITE(*,*)'Number of threads allocated this run: ',kmax
  END IF


!--------------------------------
! determine particle mass 

  numpar = 0                ! active number of particles 

  IF(maxpar.EQ.0)THEN
!    estimate the maximum particle number based upon a simple Gaussian
!    particle_density = 1.0/(0.5*(2.0*PI)**3/2*SIGX*SIGY*SIGZ)
!    where K=5.0, SIGZ=SQRT(2*K*t), SIGX=SIGY=0.5*t(sec)

     parage=crun*3600.0
     volume=conlvl*27000.0*27000.0                   
     parden=1.0/(1.97*parage*parage*SQRT(10.0*parage))
     maxpar=INT(60.0*emdur/(volume*parden)/dte)
     WRITE(*,*)'Maxpar calculated: ',maxpar
  END IF

  IF(traj)THEN
     paremt = 1
  ELSE
     numemp = 60*emdur/dte  ! number of emission periods of length dte
     paremt = maxpar/numemp ! particles per emission period

!    the number of particles in the integration loop should be evenly
!    divisible by the number of threads allocated to the calculation

     IF(MOD(paremt,kmax).NE.0)THEN
        paremt=kmax*(1+paremt/kmax)
        maxpar=paremt*numemp
     END IF
  END IF
  pmassv=totrel/maxpar      ! mass assigned to each particle


!--------------------------------
! various memory allocation sections

  ALLOCATE(xd(nxp,nyp),yd(nxp,nyp),glat(nxp,nyp),glon(nxp,nyp),STAT=kret)
  IF(kret.ne.0)THEN
     ECODE='xd,yd,glat,glon - spacing and grid position array'
     WRITE(*,*)'FATAL ERROR (memory allocation): ',ECODE,KRET
     STOP 900
  END IF

  ALLOCATE(zd(nzp),zt(nzp),zk(kzmax),hh(nxp,nyp,nzp), STAT=kret)
  IF(kret.ne.0)THEN
     ECODE='zd,zt,zk,hh - vertical grid spacing array'
     WRITE(*,*)'FATAL ERROR (memory allocation): ',ECODE,KRET
     STOP 900
  END IF

  ALLOCATE(u(nxp,nyp,nzp,2),v(nxp,nyp,nzp,2),w(nxp,nyp,nzp,2),STAT=kret)
  IF(kret.ne.0)THEN
     ECODE='u,v,w - velocity data array'
     WRITE(*,*)'FATAL ERROR (memory allocation): ',ECODE,KRET
     STOP 900
  END IF

  ALLOCATE(p(nxp,nyp,nzp,2),t(nxp,nyp,nzp,2),q(nxp,nyp,nzp,2),  &
           e(nxp,nyp,nzp,2),STAT=kret)
  IF(kret.ne.0)THEN
     ECODE='p,t,q,e - thermodynamic data array'
     WRITE(*,*)'FATAL ERROR (memory allocation): ',ECODE,KRET
     STOP 900
  END IF

  ALLOCATE(zi(nxp,nyp,2),rt(nxp,nyp,2),uf(nxp,nyp,2),hf(nxp,nyp,2),  &
           ds(nxp,nyp,2),sp(nxp,nyp,2),STAT=kret)
  IF(kret.ne.0)THEN
     ECODE='zi,rt,uf,hf,ds,sp - surface data array'
     WRITE(*,*)'FATAL ERROR (memory allocation): ',ECODE,KRET
     STOP 900
  END IF

  ALLOCATE(uu(nxp,nyp,nzp,2),vv(nxp,nyp,nzp,2),ww(nxp,nyp,nzp,2),STAT=kret)
  IF(kret.ne.0)THEN
     ECODE='uu,vv,ww - turbulence data array'
     WRITE(*,*)'FATAL ERROR (memory allocation): ',ECODE,KRET
     STOP 900
  END IF

  ALLOCATE(muu0(nxp,nyp),muv0(nxp,nyp),msfu(nxp,nyp),msfv(nxp,nyp), &
           msft(nxp,nyp),STAT=kret)
  IF(kret.ne.0)THEN
     ECODE='mf,mu - WRF time averaged flux data array'
     WRITE(*,*)'FATAL ERROR (memory allocation): ',ECODE,KRET
     STOP 900
  END IF

  nxc=1+cgratio*(nxp-1)
  nyc=1+cgratio*(nyp-1)
  ALLOCATE(cs(nxc,nyc,nzp),ct(nxc,nyc,nzp),mass(nzp),STAT=kret)
  IF(kret.ne.0)THEN
     ECODE='cs,mass - concentration array'
     WRITE(*,*)'FATAL ERROR (memory allocation): ',ECODE,KRET
     STOP 900
  END IF

  ALLOCATE(pa(16,maxpar),STAT=kret)
  IF(kret.ne.0)THEN
     ECODE='particle array'
     WRITE(*,*)'FATAL ERROR (memory allocation): ',ECODE,KRET
     STOP 900
  END IF

!--------------------------------------
! generate a large number of Gaussian random numbers

  ALLOCATE(rannum(maxran),STAT=kret)
  DO nt=1,maxran
     rannum(nt)=r4_normal_01(iseed)
  END DO

! random initialize of random number array pointer index
  IF(fixed)THEN
!    fixed initialize of random number array pointer index
     nrand=1
  ELSE
     RVALUE=r4_uniform_01(iseed)
     nrand=MIN(maxran,MAX(1,INT(rvalue*maxran)))
  END IF

! precompute autocorrelations and products for each second 
! from one to the maximum time step of "dta"

! autocorrelations
  ALLOCATE(rautoh(nbinh,dta),STAT=kret)
  ALLOCATE(rautos(nbinv,dta),STAT=kret)
  ALLOCATE(rautou(nbinv,dta),STAT=kret)

! autocorrelation products
  ALLOCATE(pautoh(nbinh,dta),STAT=kret)
  ALLOCATE(pautos(nbinv,dta),STAT=kret)
  ALLOCATE(pautou(nbinv,dta),STAT=kret)

! insure that rauto is not too small (20 = 2x10^-9)
! avoids arithmethic errors in some situations
  DO nt=1,dta
     DO nb=1,nbinv
        rautoh(nb,nt)=EXP(-MIN(20.0,nt/hscale(nb))) 
        pautoh(nb,nt)=SQRT(1.0-rautoh(nb,nt)*rautoh(nb,nt)) 
     END DO
     DO nb=1,nbinv
        rautos(nb,nt)=EXP(-MIN(20.0,nt/vscales(nb))) 
        rautou(nb,nt)=EXP(-MIN(20.0,nt/vscaleu(nb))) 
        pautos(nb,nt)=SQRT(1.0-rautos(nb,nt)*rautos(nb,nt)) 
        pautou(nb,nt)=SQRT(1.0-rautou(nb,nt)*rautou(nb,nt)) 
     END DO
  END DO

!--------------------------------------
! open the meteorological data file, define grid, start time

  CALL METPRJ (kunit,diag,gbase,datebgn)

! the ARL packed meteorological files do not define the century
! therefore we assume the century is the same as in datecal

  datebgn(1)=(datecal(1)/100)*100+datebgn(1)
  IF(diag) WRITE(*,'(A,6I4)')'Meteorology start: ',datebgn
  IF(diag) WRITE(*,'(A,6I4)')'Calculation start: ',datecal

! initialize variables for output
  IF(traj) THEN
     CALL TRJSET (munit,back,olat,olon,olvl,datebgn,datecal,trjfile)

  ELSE
     IF(datem)THEN
!       output in DATEM format reads measured data first
        CALL DATSET (junit,gbase,datfile,century)
        IF(century.NE.(datecal(1)/100)*100)THEN
           WRITE(*,*)'DATEM input file century = ',century
           WRITE(*,*)'SETUP.CFG starts century = ',(datecal(1)/100)*100
           STOP
        END IF
     ELSE
        IF(hybin)  &   
           CALL CONSET (junit,olat,olon,olvl,datebgn,datecal,confile)
     END IF
  END IF

! compute the number of minutes after the computational
! start time CSEC, that the particle emissions RSEC will start

  CSEC =JULSEC(datecal(1),datecal(2),datecal(3),datecal(4),datecal(5),0)
  RSEC =JULSEC(daterel(1),daterel(2),daterel(3),daterel(4),daterel(5),0)
  pemit=(RSEC-CSEC)/60
  IF(diag) WRITE(*,'(A,I4)')'Emissions relative start (min): ',pemit

!--------------------------------------

! particles starting position for integration
  CALL CLL2XY(GBASE,OLAT,OLON,X0,Y0)
  Z0=OLVL ! meters AGL

  jet=0  ! current elapsed time from calculation start (sec)

!---------------------------------------------

! compute the starting record number as the difference between the first 
! time period in the file BSEC and the computational start time CSEC  

  BSEC =JULSEC(datebgn(1),datebgn(2),datebgn(3),datebgn(4),datebgn(5),0)
  DELTA=CSEC-BSEC
  krec=numper*(DELTA/DELM)

  krec=krec+1            ! set record pointer to index record
  ntp=1+crun*3600/delm   ! number of meteorology times to process

  DO nt=1,ntp  

     IF(diag)WRITE(*,'(A,3I6)')'Period,Time: ',nt,jet

!    analyze the index record of each time period
     READ(KUNIT,REC=KREC,IOSTAT=KRET)LABEL,HEADER

     IF(KRET.NE.0)THEN
!       assume any error means reading beyond end of file
        CLOSE(KUNIT)
        mfile=mfile+1
        krec=1
        WRITE(*,*)'Opening: ',TRIM(metfile(mfile))
        INQUIRE(FILE=metfile(mfile),EXIST=FTEST)
        IF(.NOT.FTEST)THEN
           WRITE(*,*)'File not found: ',TRIM(metfile(mfile))
           STOP
        END IF
        OPEN(KUNIT,FILE=METFILE(mfile),STATUS='OLD',ACTION='READ', &
             RECL=LREC,ACCESS='DIRECT',FORM='UNFORMATTED')
        READ(KUNIT,REC=KREC)LABEL,HEADER
     END IF

     READ(LABEL,'(5I2,4X,A4)') dateinp(1:4),dateinp(6),KVAR
     READ(HEADER,'(7X,I2)')dateinp(5)
     dateinp(1)=(datecal(1)/100)*100+dateinp(1)
     IF(KVAR.NE.'INDX')THEN
        WRITE(*,*)'First record in time period not equal to index!'
        WRITE(*,*)KREC,'  ',LABEL
        STOP
     END IF
     IF(diag) WRITE(*,'(A,4I4)')'Input:   ',dateinp(1:4)

!    advance record pointer to the corresponding data record
     KREC=KREC+NUMNDX  

!    insure that meteorology input matches expected date-time
     RSEC=JULSEC(dateinp(1),dateinp(2),dateinp(3),dateinp(4),dateinp(5),0)
     DINP=JET+DELM
     IF(BACK)DINP=-DINP  

     IF(RSEC-CSEC.NE.DINP.AND.NT.GE.2)THEN
        WRITE(*,*)'Meteorology input date-time does not match expected'
        WRITE(*,*)'From meteorol label: ',dateinp
        WRITE(*,*)'Current offset time: ',DINP/60
        WRITE(*,*)'Meterol offset time: ',(RSEC-CSEC)/60
!       STOP
     END IF

!    Set time variables for sampling start and stop times
!    Start and stop times must correspond with meteorology
     IF(nt.EQ.1)THEN
        IF(datem) CALL DATSUM(dateinp,cs)
        dateprv=dateinp
     END IF

!    set sampling start date time
     IF(jet.EQ.cstart)THEN
        datebgn=dateprv
        CS=0.0              ! initialize particle mass sum 
     END IF

!--------------------------------------------
!    input meteorological data; parallelization in reading the
!    upper level data (each level to its own core) does not
!    work because of thread conflicts with the I/O buffer

     CALL METSFC(KUNIT,K2,KREC,NVSFC)
     KREC=KREC+NVSFC ! point to first level record

! PARALLEL SHARED(kunit,k2,krec,nzp,nvlvl)
! DO PRIVATE(nz,jrec)

     DO nz=2,nzp
        JREC=KREC+(NZ-2)*NVLVL ! compute 1st record for level=NZ
        CALL METLVL(KUNIT,K2,JREC,NVLVL,TAVG)
     END DO

! END DO
! END PARALLEL

!    when all upper levels completed, point to next index record
     IF(BACK)THEN
        KREC=KREC+(NZP-1)*NVLVL  
        KREC=KREC-2*numper
     ELSE
        KREC=KREC+(NZP-1)*NVLVL  
     END IF

!    convert the input meteorology to the proper units for
!    advection and dispersion calculations

!##  note that the COLLAPSE statement may not work with some compilers,
!##  in that case just enable the parallel jj loop

!$OMP PARALLEL SHARED(tked,k2,tker,ptop,sg)
!$OMP DO PRIVATE(ii,jj) COLLAPSE(2)

     DO ii=1,nxp
     DO jj=1,nyp
        CALL METCNV(tked,ii,jj,k2,tker,ptop,sg)
     END DO
     END DO

!$OMP END DO
!$OMP END PARALLEL

!    set position for grid index
     IF(ptop.GE.0.0)THEN
        ii=INT(x0)
        jj=INT(y0)
     ELSE
        ii=0
        jj=0
     END IF
     
     IF(zgrid)THEN
!       configure the vertical grid after each meteorology read
        CALL METGRD(ii,jj)            
     ELSE
!       only configure after the first meteo read (k1=0 k2=1)
        IF(K1.EQ.0) CALL METGRD(ii,jj)            
     END IF

     IF(diag) CALL METPRT (INT(x0),INT(y0),k2)

!--------------------------------------------
!    advection-dispersion starts only after 2 meteo data periods loaded
     IF(NT.GE.2)THEN  

  ns=1
  dt=dte     ! assume calculation starts with emissions on

! number of time steps per meteo data interval
  DO WHILE (ns.LE.delm/dt)

     EMIT=.FALSE.
     IF(jet/60.GE.pemit.AND.jet/60.LT.pemit+emdur) EMIT=.true.
     IF(traj.AND.jet.GT.0) EMIT=.FALSE.

!    emission section
     IF(EMIT)THEN
!       emissions ON time step is set to dte
        dt=dte

!       Particle emissions assigns mass, position, and turbulence values;
!       initialize turbulence from random array; however for particles
!       released at the same time, the turbulence initializaion should be
!       the same and for particles released after some delta-T assume the
!       autocorrelation is zero because the release is at a fixed point
!       and not following the particle.

        UP0=0.0
        VP0=0.0
        WP0=0.0
        WR0=0.0

!       force the time step to its maximum (DTA) to keep the autocorrelation 
!       near zero which means that the turbulence values will be close to 
!       random for each release time (rauto=0.0 and pauto=1.0) and the value 
!       of DELZ (=100) doesn't matter because WP0=0 which eliminates the 
!       SIGDZ term; also the particle number is forced to 1 which gives the 
!       smallest VSCALE bin and in conjunction with large DT a rauto near 0

        ZF=ZK(INT(Z0))
        CALL PARDSP(K1,K2,0.0,DTA,1 ,NRAND,100.,X0,Y0,ZF,UP0,VP0,WP0,WR0)

        DO np=(numpar+1),(numpar+paremt)
!          particle mass
           PA(1,NP)=PMASSV         

!          particle position
           PA(2,NP)=X0 
           PA(3,NP)=Y0
           PA(4,NP)=Z0

!          all particles released at the same time start with the 
!          same initial turbulence, but with a different index
           PA(5,NP)=UP0 
           PA(6,NP)=VP0
           PA(7,NP)=WP0
           PA(8,NP)=WR0
           PA(9,NP)=FLOAT(NP)
           NRAND=MOD(NRAND+3,MAXRAN)+1
           PA(10,NP)=FLOAT(NRAND)
        END DO

        IF(traj)THEN
           dt=dta
!          for trajectories output the starting location
           CALL CXY2LL(GBASE,PA(2,1),PA(3,1),TLAT,TLON)
           TZHT=PA(4,1)
           ZF=ZK(MAX(1,MIN(NINT(tzht),KZMAX)))
           CALL ADV3NT(P(:,:,:,K1),PA(2,1),PA(3,1),ZF,tmetv)
           CALL TRJDSK(munit,datecal,(jet/3600.0),tlat,tlon,tzht,tmetv)
        END IF

!       increase computational particle counter
        NUMPAR=NUMPAR+PAREMT  
        IF(diag)WRITE(*,'(A,2I6)')'Emit: ',jet,numpar

     ELSE
!       emissions OFF time step is set to dta
        dt=dta
     END IF

!--------------------------------------------

!$OMP PARALLEL SHARED(numpar,k1,k2,delm,jet,dt,traj,back,pa) 
        CT=0.0
!$OMP DO PRIVATE(np,kthr) REDUCTION(+:ct) 
        DO np=1,numpar
           kthr=OMP_GET_THREAD_NUM() ! active thread
           CALL ADVIEC(K1,K2,DELM,JET,DT,TRAJ,TAVG,BACK,CT,PA(:,NP))
        END DO
!$OMP END DO
!$OMP END PARALLEL
        CS=CS+CT

     jet=jet+dt
     ns=ns+1
  END DO  !  time steps per meteo data interval

!--------------------------------------------

!    single trajectory output section
     IF(traj.AND.MOD(JET,3600).EQ.0)THEN
     IF(PA(2,1).GT.0.0)THEN
        dateend=dateinp
        CALL CXY2LL(GBASE,PA(2,1),PA(3,1),TLAT,TLON)
        TZHT=PA(4,1)
        ZF=ZK(MAX(1,MIN(NINT(tzht),KZMAX)))
        CALL ADV3NT(P(:,:,:,K1),PA(2,1),PA(3,1),ZF,tmetv)
        HOURS=jet/3600.0
        IF(back)HOURS=-HOURS
        CALL TRJDSK(munit,dateend,hours,tlat,tlon,tzht,tmetv)
!       for diagnostic ouput (metprt)
        X0=PA(2,1)
        Y0=PA(3,1)
     END IF
        datebgn=dateend
     END IF

!--------------------------------------------

     IF(.NOT.traj)THEN
!    output concentrations at the end of the averaging period
!    which needs to corrrespond with one of the meteorology times
     IF(jet.GT.cstart.AND.MOD(jet-cstart,cinter).EQ.0)THEN
        dateend=dateinp
        IF(diag)WRITE(*,'(A,I6,2X,4I5)')'Sample end: ',jet,dateend(1:4)

!       compute average cell mass
        CS=CS/cinter   

        IF(datem)THEN
!          check all samplers and accumulate concentrations until
!          the end sampling at each location; output end of simulation
           CALL DATSUM(dateend,cs)
        ELSE
!          write to output file at the interval of cinter
           IF(hybin) &   
              CALL CONOUT(junit,datebgn,dateend,gbase,cfactor,cs)
           IF(html)  &  
              CALL SVGOUT(nunit,iunit,svgfile,mapfile,datebgn,dateend, &
                          thin,x0,y0,gbase,cfactor,cs)
        END IF

!       diagnostic mass dump
        IF(diag)THEN

!          total mass of all particles
           mass=0.0
           DO np=1,numpar
              ZF=ZK(MAX(1,NINT(PA(4,np))))
              KK=MIN(INT(ZF)+1,NZP)
              IF(pa(2,np).GT.0.0) mass(kk)=mass(kk)+pa(1,np)
           END DO
           tmass=SUM(mass)

!          print mass profile
           IF(tmass.GT.0.0)THEN
              WRITE(*,*)
              WRITE(*,'(10X,A3,A7,A10)')'K','Height','Percent'
              DO KK=NZP,1,-1   ! print alternate levels
                WRITE(*,'(10X,I3,I7,F10.1)')KK,NINT(ZT(kk)),100.0*MASS(KK)/tmass
              END DO
           END IF

           WRITE(*,*)'      Total Mass: ',tmass
           WRITE(*,*)
        END IF ! diagnostic mass dump

!       reinitialize for next averaging period
        CS=0.0

!       sample start equals sample stop
        datebgn=dateend

     END IF  !  concentration averaging period
     END IF  !  not a trajectory section
     END IF  !  dispersion section NT>=2 end 

!--------------------------------------------

!    cycle time index for next meteorology input
     dateprv=dateinp  
     K1=K2
     K2=MOD(K2,2)+1

     IF(.NOT.diag)THEN
        IF(nt.EQ.INT(0.25*ntp)) WRITE(*,*)'#---'
        IF(nt.EQ.INT(0.50*ntp)) WRITE(*,*)'##--'
        IF(nt.EQ.INT(0.75*ntp)) WRITE(*,*)'###-'
        IF(nt.EQ.ntp)           WRITE(*,*)'####'
     END IF

! number of input time periods
  END DO

! write DATEM file with measured data replaced by calculated 
  IF(datem) CALL DATOUT(junit,cfactor,olat,olon,stnfile)

! check for particle array consistency
! when running in a multi-thread environment
  IF(kmax.gt.1)THEN
     np=1
     DO WHILE (np.LE.numpar)
        IF(INT(pa(9,np)).NE.np) &    
           WRITE(*,*)'Particle array error at [pa(9)<>np]: ',pa(9,np),np
        np=np+1
     END DO
  END IF

! free memory 
  DEALLOCATE(u,v,w,p,t,q,e,uu,vv,ww, STAT=kret)
  IF(kret.NE.0) WRITE(*,*)'deallocation #1 error: ',kret
  DEALLOCATE(hh,xd,yd,zd,zt,zk,glat,glon, STAT=kret)
  IF(kret.NE.0) WRITE(*,*)'deallocation #2 error: ',kret
  DEALLOCATE(rannum, STAT=kret)
  IF(kret.NE.0) WRITE(*,*)'deallocation #3 error: ',kret
  DEALLOCATE(pa, STAT=kret)
  IF(kret.NE.0) WRITE(*,*)'deallocation #4 error: ',kret
  DEALLOCATE(mass,cs,ct, STAT=kret)
  IF(kret.NE.0) WRITE(*,*)'deallocation #5 error: ',kret
  DEALLOCATE(rautos,rautou,rautoh, STAT=kret)
  IF(kret.NE.0) WRITE(*,*)'deallocation #6 error: ',kret
  DEALLOCATE(pautos,pautou,pautoh, STAT=kret)
  IF(kret.NE.0) WRITE(*,*)'deallocation #7 error: ',kret
  DEALLOCATE(msfu,msfv,msft,muu0,muv0, STAT=kret)
  IF(kret.NE.0) WRITE(*,*)'deallocation #8 error: ',kret

!--------------------------------
  CALL DATE_AND_TIME(REAL_CLOCK(1),REAL_CLOCK(2),REAL_CLOCK(3),DATE_TIME)
  READ(REAL_CLOCK(1),'(I4,2I2)')YR,MO,DA   
  READ(REAL_CLOCK(2),'(2I2,F6.3)')HR,MN,SS
  DSEC=JULSEC(YR,MO,DA,HR,MN,INT(SS))-DSEC
  WRITE(*,*)'Elapsed time (s) = ',DSEC
!--------------------------------

END PROGRAM adapt   

INCLUDE 'pakinp.f'
INCLUDE 'mapbdy.f'
INCLUDE 'metlvl.f'
INCLUDE 'metgrd.f'
INCLUDE 'metset.f'
INCLUDE 'metprj.f'
INCLUDE 'metprt.f'
INCLUDE 'metsfc.f'
INCLUDE 'metcnv.f'
INCLUDE 'adviec.f'
INCLUDE 'adv3nt.f'
INCLUDE 'advpnt.f'
INCLUDE 'advdel.f'
INCLUDE 'normal.f'
INCLUDE 'pardsp.f'
INCLUDE 'conset.f'
INCLUDE 'conout.f'
INCLUDE 'datset.f'
INCLUDE 'datsum.f'
INCLUDE 'datout.f'
INCLUDE 'julsec.f'
INCLUDE 'svgout.f'
INCLUDE 'trjset.f'
INCLUDE 'trjdsk.f'
