!-----------------------------------------------------------------------------
! Primary advection subroutine that uses the predictor-corrector method,
! also known as the Improved Euler-Cauchy (IEC) integration, uses linear
! interpolation to compute the meteorological variables at the x,y,z,t 
! particle positions. The turbulent dispersion displacement is added to the
! position each time step. Stability criteria of a displacement not to exceed
! tratio of a grid cell size is applied only to the first position estimate. 
! If exceeded, then the internal time step is recalculated.
! ---------------------------------------------------------------------------- 
! Last Revised: 31 Jul 2020 - correction to PA array
!               09 Aug 2020 - eliminated vertical in time step criteria
!               30 Oct 2020 - added trajectory flag
!               19 Nov 2020 - test for time averaged data
!               15 Jan 2021 - integration direction
! ---------------------------------------------------------------------------- 

SUBROUTINE ADVIEC(K1,K2,DELM,JET,DELT,TRAJ,TAVG,BACK,CS,PA) 

  USE metval, ONLY: zt,zk,zd
  USE ranval

  IMPLICIT NONE

!-----------------------------------------------------------------
! argument list definitions

  INTEGER, INTENT(IN)    :: k1,k2           ! last and next time index    
  INTEGER, INTENT(IN)    :: delm            ! seconds between meteo indicies
  INTEGER, INTENT(IN)    :: jet             ! current elapsed time (sec)
  INTEGER, INTENT(IN)    :: delt            ! default integration step (sec)
  LOGICAL, INTENT(IN)    :: traj            ! flag to turn off dispersion
  LOGICAL, INTENT(IN)    :: tavg            ! time averaged meteorology
  LOGICAL, INTENT(IN)    :: back            ! integration direction
  REAL,    INTENT(INOUT) :: cs(:,:,:)       ! concentration summation
  REAL,    INTENT(INOUT) :: pa(:)           ! particle array     

!------------------------------------------------------------------
! internal variables

  REAL     :: tf                            ! time interpolation factor
  REAL     :: zf                            ! fractional vertical position
  REAL     :: delh                          ! vertical cell depth
  REAL     :: zref                          ! top boundary for reflection
  
  INTEGER  :: dt                            ! computational time step (s)
  INTEGER  :: ii,jj,kz                      ! indicies
  INTEGER  :: ksec,knum                     ! internal time and counters
  LOGICAL  :: reset                         ! flag to reset time step

  INTEGER  :: nxp,nyp,nzp                   ! grid limits
  INTEGER  :: kzmax                         ! model top (m)
  INTEGER  :: cgratio                       ! numb of conc cells to WRF

  REAL     :: PM,XP,YP,ZP,UP,VP,WP,WR       ! particle array values for one
  INTEGER  :: nrand                         ! index pointer for random number
  INTEGER  :: np                            ! sequential particle number

!-------------------------------------------------------------------

  INCLUDE 'adviec.inc'

!-------------------------------------------------------------------

  COMMON /gridpt/ nxp,nyp,nzp,kzmax,cgratio

!-------------------------------------------------------------------

  PM=PA(1)
  XP=PA(2)
  YP=PA(3)
  ZP=PA(4)
  UP=PA(5)
  VP=PA(6)
  WP=PA(7)
  WR=PA(8)
  NP=INT(PA(9))
  NRAND=INT(PA(10))

  IF(XP.LT.0.0)RETURN  ! x,y,z positions <0 are offgrid
  ZREF=ZT(nzp)         ! top height of upper-most grid cell

  KSEC=JET             ! internal elapsed time
  DT=DELT              ! initial time step is the default time step
  RESET=.FALSE.        ! only reset integration DT one time

  KNUM=1
  DO WHILE (KNUM.LE.DELT/DT)

!    determine interpolation time factor for the current
!    time (ksec) between two meteorology times (dm)
     TF=MOD(KSEC,DELM)/FLOAT(DELM)

     IF(tavg)THEN
!       time-averaged winds assume persistence of the
!       K2 time period data between K1 and K2
        TF=1.0
        IF(back)TF=-1.0
     END IF

!    first pass position update
     CALL ADVPNT(K1,K2,DELT,DT,TF,KNUM,TRAJ,BACK,RESET,XP,YP,ZP,UP,VP,WP)
     IF(XP.LT.0.0)RETURN

!    reset will be true if delt<>dt and therefore recompute first pass
     IF(RESET.AND.KNUM.EQ.1)THEN
        XP=PA(2)
        YP=PA(3)
        ZP=PA(4)
        CALL ADVPNT(K1,K2,DELT,DT,TF,KNUM,TRAJ,BACK,RESET,XP,YP,ZP,UP,VP,WP)
        IF(XP.LT.0.0)RETURN
     END IF

!    advance internal time (T+deltaT) for the next interpolation
     KSEC=KSEC+DT
     TF=MOD(KSEC,DELM)/FLOAT(DELM)
     IF(tavg)THEN
        TF=1.0
        IF(back)TF=-1.0
     END IF

!    the final pass is the average of the 1st pass end position and the 2nd pass 
!    position. The 2nd pass uses the U,V,W at the end position of the 1st pass 
!    where XF=0.5(X1+X0+DX), and DX=X2-X1, therefore XF=0.5(X0+X2)

     CALL ADVPNT(K1,K2,DELT,DT,TF,KNUM,TRAJ,BACK,RESET,XP,YP,ZP,UP,VP,WP)
     IF(XP.LT.0.0)RETURN
     XP=0.5*(PA(2)+XP)                               
     YP=0.5*(PA(3)+YP)
     ZP=0.5*(PA(4)+ZP)

!    update particle turbulence variances at the final position
     ZF=ZK(MAX(1,MIN(NINT(ZP),kzmax))) 
     DELH=ZD(MIN(INT(ZF)+1,NZP))
     CALL PARDSP(K1,K2,TF,DT,NP,NRAND,DELH,XP,YP,ZF,UP,VP,WP,WR)
     NRAND=MOD(NRAND+3,MAXRAN)+1   ! because there are three calls in pardsp

!    accumulate mass to concentration array weighted by the time step
!    because the time step may vary during the averaging period, and
!    count particles to the top of each cell; convert WRF grid x,y
!    position to concentration grid multiple 

     II=NINT(1.0+CGRATIO*(XP-1.0))
     JJ=NINT(1.0+CGRATIO*(YP-1.0))
     KZ=MIN(INT(ZF)+1,NZP)
     CS(ii,jj,kz)=CS(ii,jj,kz)+PM*ABS(DT)

!    update particle array after each step
     PA(1)=PM
     PA(2)=XP
     PA(3)=YP
     PA(4)=ZP
     PA(5)=UP
     PA(6)=VP
     PA(7)=WP
     PA(8)=WR
     PA(10)=FLOAT(NRAND)

     KNUM=KNUM+1
  END DO 

END SUBROUTINE adviec
