!-----------------------------------------------------------------------------
! Single pass of the advection subroutine for a single point which uses linear
! interpolation to compute the meteorological variables at the x,y,z,t 
! particle positions. The turbulent dispersion displacement is added to the
! position each time step. Stability criteria of a displacement not to exceed
! tratio of a grid cell size is applied only to the first position estimate. 
! If exceeded, then the internal time step is recalculated.
! ---------------------------------------------------------------------------- 
! Last Revised: 02 Nov 2020 - adapted from adviec    
!               15 Jan 2021 - added back option
! ---------------------------------------------------------------------------- 

SUBROUTINE ADVPNT(K1,K2,DELT,DT,TF,KNUM,TRAJ,BACK,RESET,XP,YP,ZP,UP,VP,WP) 

  USE metval

  IMPLICIT NONE

!-----------------------------------------------------------------
! argument list definitions

  INTEGER, INTENT(IN)    :: k1,k2           ! last and next time index    
  INTEGER, INTENT(IN)    :: delt            ! default integration step (sec)
  INTEGER, INTENT(INOUT) :: dt              ! adjusted integration step (sec)
  REAL,    INTENT(IN)    :: tf              ! time interpolation factor
  INTEGER, INTENT(IN)    :: knum            ! integration counter
  LOGICAL, INTENT(IN)    :: traj            ! flag to turn off dispersion
  LOGICAL, INTENT(IN)    :: back            ! integration direction       
  LOGICAL, INTENT(INOUT) :: reset           ! flag to reset time step
  REAL,    INTENT(INOUT) :: xp,yp,zp        ! particle position
  REAL,    INTENT(INOUT) :: up,vp,wp        ! turbulent velocities

!------------------------------------------------------------------
! internal variables

  REAL     :: uut,vvt,wwt        ! time interpolation- at jet  
  REAL     :: uu1,vv1,ww1        ! time interpolation- previous          
  REAL     :: uu2,vv2,ww2        ! time interpolation- next

  REAL     :: zf                 ! fractional vertical position
  REAL     :: zref               ! top boundary for reflection
  REAL     :: zpbl               ! boundary layer depth
  REAL     :: dirdt              ! direction adjusted time step
  REAL     :: delx,dely,delz     ! advection distance per time step
  REAL     :: SUX,SUY,SVX,SVY    ! grid stagger variables
  
  INTEGER  :: nxp,nyp,nzp        ! grid limits
  INTEGER  :: kzmax              ! model top (m)
  INTEGER  :: cgratio            ! number of concentration cells to WRF
  INTEGER  :: ii,jj              ! indicies

  REAL     :: tratiov,tratioh    ! ratio for time step stability
  REAL     :: tkemax             ! maximum value turbulent kinetic energy (J/kg)
  REAL     :: tkemin             ! minimum value turbulent kinetic energy (J/kg)
  REAL     :: pblmin             ! minimum value for the PBL depth (m)

!-------------------------------------------------------------------

  INCLUDE 'advpnt.inc'

!-------------------------------------------------------------------

  COMMON /gridpt/ nxp,nyp,nzp,kzmax,cgratio
  COMMON /offset/ sux,suy,svx,svy
  COMMON /stblty/ tratiov,tratioh,tkemin,tkemax,pblmin

!-------------------------------------------------------------------

  IF(XP.LT.0.0)RETURN  ! x,y,z positions <0 are offgrid
  ZREF=ZT(nzp)         ! top height of upper-most grid cell

! Before any interpolation check the position values against limits. For the
! horizontal the index is the grid center. In the vertical the index is the top 
! of the cell, one is the ground surface. Particles reflect at the boundaries; 
! note that the real vertical grid ranges from one to (nzp-1) for reflection.  

  II=NINT(XP)
  JJ=NINT(YP) 
  IF(II.LE.1.OR.II.GE.NXP-1.OR.  &
     JJ.LE.1.OR.JJ.GE.NYP-1)  THEN
     XP=-1.0
     RETURN
  END IF

! fractional grid index
  ZF=ZK(MAX(1,MIN(NINT(ZP),kzmax)))

! interpolate winds to the position at the previous time
  CALL ADV3NT(U(:,:,:,K1),XP+SUX,YP+SUY,ZF,UU1)
  CALL ADV3NT(V(:,:,:,K1),XP+SVX,YP+SVY,ZF,VV1)
  CALL ADV3NT(W(:,:,:,K1),XP    ,YP    ,ZF,WW1)

! interpolate winds to the position at the next time
  CALL ADV3NT(U(:,:,:,K2),XP+SUX,YP+SUY,ZF,UU2)
  CALL ADV3NT(V(:,:,:,K2),XP+SVX,YP+SVY,ZF,VV2)
  CALL ADV3NT(W(:,:,:,K2),XP    ,YP    ,ZF,WW2)

! interpolate to current time
  UUT=(UU2-UU1)*TF+UU1
  VVT=(VV2-VV1)*TF+VV1
  WWT=(WW2-WW1)*TF+WW1

! Add the current mean and previous turbulent components
! (u-prime) for the displacement velocity (all units m/s).
  IF(.NOT.traj)THEN
     UUT=UUT+UP 
     VVT=VVT+VP
     WWT=WWT+WP
  END IF

! direction adjusted time step
! so that the motions u,v,w are all upwind
  IF(BACK)THEN
     DIRDT=-DT
  ELSE
     DIRDT=DT
  END IF
 
! horizontal displacement in grid units
  DELX=UUT*DIRDT/XD(II,JJ)
  DELY=VVT*DIRDT/YD(II,JJ)              
! vertical displacement in meters
  DELZ=WWT*DIRDT

! test for time-step stability (default limit to TRATIO of grid size)
! only once on the first iteration; zref/nzp = average vertical cell
  IF(.NOT.RESET.AND.KNUM.EQ.1) THEN
     CALL ADVDEL(tratiov,tratioh,delx,dely,(nzp*delz/zref),delt,dt)
     IF(dt.NE.delt) THEN
        RESET=.TRUE.  
        RETURN
     END IF
  END IF

! particle position
  XP=XP+DELX
  YP=YP+DELY
  ZP=ZP+DELZ

! test section to limit unstable mixing to within the PBL but
! only when the PBL is defined as greater than the minimum
! (this section seems to have no effect upon the results!)
  ZPBL=(ZI(II,JJ,K2)-ZI(II,JJ,K1))*TF+ZI(II,JJ,K1)  
  IF(ZP.LT.ZPBL.AND.ZPBL.GT.PBLMIN)THEN
     ZREF=ZPBL
  ELSE
     ZREF=ZT(nzp) 
  END IF 

! check for reflection at boundaries 
! turbulent velocity direction remains unaffected (WP & WR)
  IF(ZP.LT.0.0)THEN
     ZP=-ZP
  ELSEIF(ZP.GT.ZREF)THEN
     ZP=2.0*ZREF-ZP
  ELSE
     CONTINUE
  END IF

! check position values against grid limits
  IF(NINT(XP).LE.1.OR.NINT(XP).GE.NXP-1.OR.&
     NINT(YP).LE.1.OR.NINT(YP).GE.NYP-1)THEN
     XP=-1.0
     RETURN
  END IF

END SUBROUTINE advpnt
