! ----------------------------------------------------------------------------
! Map the internal WRF concentration grid to the HYSPLIT output format and
! write the data record. Primary grid is WRF so that every WRF grid point is
! mapped to the output grid. Multiple intersections to the same output grid
! the max value is used. If the output grid is too fine compared with the WRF
! grid, some output grid points may show zero values in the middle of a plume.
! ----------------------------------------------------------------------------
! Last Revised: 09 Dec 2019
!               08 Aug 2020 - moved some variables to the conval common block
!               12 Aug 2020 - moved vertical index outside of i,j loop
!               18 Nov 2020 - generalized time variable
!               28 Dec 2020 - process four digit year
!               02 Jan 2021 - concentration grids adjusted for CGRATIO
! ----------------------------------------------------------------------------

SUBROUTINE CONOUT(junit,datebgn,dateend,gbase,cfactor,cs)

  USE metval, ONLY: zt,yd,xd,glat,glon

  IMPLICIT NONE

  INTEGER, INTENT(IN) :: junit
  INTEGER, INTENT(IN) :: datebgn(6)
  INTEGER, INTENT(IN) :: dateend(6) 
  REAL,    INTENT(IN) :: gbase(9)  
  REAL,    INTENT(IN) :: cfactor
  REAL,    INTENT(IN) :: cs(:,:,:)   

  INTEGER,      PARAMETER :: nlvl=1    ! number of vertical grid levels
  INTEGER,      PARAMETER :: ntyp=1    ! number of pollutant grids
  INTEGER,      PARAMETER :: cpack=1   ! concentration packing on (off=0)

  REAL(4),    ALLOCATABLE :: CONC(:,:) ! two dimensional concentration grid
  REAL(4),    ALLOCATABLE :: CVAL(:)   ! non-zero grid values
  INTEGER(2), ALLOCATABLE :: IPTS(:)   ! grid index value in longitude 
  INTEGER(2), ALLOCATABLE :: JPTS(:)   ! grid index value in latitude

  INTEGER      :: I,J,II,JJ            ! latlon and conc grid indicies
  INTEGER      :: KK,KL,KT,KNTR,KTOP   ! local variables

  REAL         :: CTOT
  REAL         :: CGSZXY
  REAL         :: XP,YP                ! position on meteorology grid
  REAL         :: XDIS,YDIS            ! concentration grid spacing
  REAL         :: XLON,YLAT            ! concentration grid position 

! concentration grid configuration
  CHARACTER(4) :: polid        ! pollutant identification
  INTEGER      :: conlvl       ! height m-AGL of grid plane
  REAL         :: clat         ! lower left corner concengration grid
  REAL         :: clon         ! lower left corner concentration grid
  REAL         :: dlat         ! grid spacing in degrees latitude
  REAL         :: dlon         ! grid spacing in degrees longitude
  INTEGER      :: nlon         ! number of longitude grid points
  INTEGER      :: nlat         ! number of latitude grid points

  INTEGER      :: nxp,nyp,nzp,nxc,nyc 
  INTEGER      :: kzmax        ! model top (m)
  INTEGER      :: cgratio      ! number of concentration grid cells to WRF cells
  INTEGER      :: yy           ! two digit year

  COMMON /gridpt/ nxp,nyp,nzp,kzmax,cgratio
  COMMON /conval/ polid,conlvl,clat,clon,dlat,dlon,nlat,nlon

  SAVE conc,cval,ipts,jpts

!-----------------------------------------------------
  
  IF(.NOT.ALLOCATED(conc))THEN
     ALLOCATE (conc(nlon,nlat))
     ALLOCATE (cval(nlon*nlat))
     ALLOCATE (ipts(nlon*nlat),jpts(nlon*nlat))
  END IF

  CONC=0.0     ! initialize output grid

! find the top of the output layer 
  KK=2
  DO WHILE (KK.LE.NZP.AND.ZT(KK).LE.CONLVL)
     KK=KK+1      
  END DO
  KTOP=KK-1

! concentration grid limits
  nxc=1+cgratio*(nxp-1)
  nyc=1+cgratio*(nyp-1)

! Go through each point on the meteorology grid and remap to the 
! output concentration grid using nearest neighbor approach. Round the 
! position so that cell is centered over a point such that a particle
! within grid cell #1 has a real position value of 0.5 to 1.499.

  DO II=1,NXC
  DO JJ=1,NYC

     CTOT=SUM(CS(II,JJ,2:KTOP))

!    process only non-zero grid points
     IF(CTOT.GT.0.0)THEN

        IF(cgratio.EQ.1)THEN
!          use pre-computed meteorology grid conversions
           YLAT=GLAT(II,JJ)
           XLON=GLON(II,JJ)
           XDIS=XD(II,JJ)
           YDIS=YD(II,JJ)

        ELSE
!          convert concentration grid point to meteorology grid
           XP=1.0+(II-1.0)/CGRATIO
           YP=1.0+(JJ-1.0)/CGRATIO

!          recompute position and resolution
           CALL CXY2LL(GBASE,XP,YP,YLAT,XLON)
!          meters per grid cell
           XDIS=CGSZXY(GBASE,XP,YP)*1000.0
           YDIS=XDIS
        END IF

!       remap to the lat-lon output grid
        I=NINT(1.0+(XLON-CLON)/DLON)
        J=NINT(1.0+(YLAT-CLAT)/DLAT)
        IF(I.GE.1.AND.J.GE.1.AND.I.LE.NLON.AND.J.LE.NLAT)THEN
!          vertically integrated cell volume
           CTOT=CTOT*CGRATIO*CGRATIO/(XDIS*YDIS*ZT(KTOP))
!          update output grid
           CONC(I,J)=MAX( CONC(I,J), CTOT)   
        END IF

     END IF
  END DO
  END DO
 
!-----------------------------------------------------

! write record for sampling begin time
  yy=datebgn(1)-(datebgn(1)/100)*100
  WRITE(JUNIT)yy,datebgn(2:6)

! write record for sampling end time
  yy=dateend(1)-(dateend(1)/100)*100
  WRITE(JUNIT)yy,dateend(2:6) 

! write concentration grid records by pollutant type and level
  DO KT=1,NTYP
  DO KL=1,NLVL

!    only write non-zero concentration grid points when packing
     IF(CPACK.EQ.1)THEN
        KNTR=0
        DO JJ=1,NLAT
        DO II=1,NLON

           IF(CONC(II,JJ).GT.0.0)THEN
              KNTR=KNTR+1
              IPTS(KNTR)=INT(II,KIND=2)
              JPTS(KNTR)=INT(JJ,KIND=2)
              CVAL(KNTR)=CONC(II,JJ)*CFACTOR
           END IF

        END DO 
        END DO

        IF(KNTR.GT.0)THEN
!          one or more non-zero concentration grid points
           WRITE(JUNIT)polid,conlvl,KNTR,(IPTS(KK),JPTS(KK),CVAL(KK),KK=1,KNTR)
        ELSE
!          all grid points are zero 
           WRITE(JUNIT)polid,conlvl,KNTR
        END IF

     ELSE
!       write all grid points (zero and nonzero)
        WRITE(JUNIT)polid,conlvl, ((CONC(II,JJ),II=1,NLON),JJ=1,NLAT)
     END IF

  END DO
  END DO

END SUBROUTINE conout   
