! ----------------------------------------------------------------------------
! Configure the output concentration grid variables and open the file and
! write the header records: follows the HYSPLIT binary concentration file.
! ----------------------------------------------------------------------------
! Last Revised: 09 Dec 2019
!               08 Aug 2020 - moved some variables to the conval common block
!               17 Nov 2020 - generalized grid definitions
!               28 Dec 2020 - configure to handle 4-digit year input 
!               02 Jan 2021 - option to get finer grid using CGRATIO
! ----------------------------------------------------------------------------

SUBROUTINE CONSET (junit,olat,olon,olvl,datemet,datecal,confile)  

  IMPLICIT NONE

  INTEGER,        INTENT(IN) :: junit
  REAL,           INTENT(IN) :: olat       ! pollutant source latitude
  REAL,           INTENT(IN) :: olon       ! pollutant source longitude
  REAL,           INTENT(IN) :: olvl       ! pollutant source height
  INTEGER,        INTENT(IN) :: datemet(6) ! meteo data start (yymmddhhmmff)
  INTEGER,        INTENT(IN) :: datecal(6) ! calculation start (yymmddhhmmff)
  CHARACTER(256), INTENT(IN) :: confile    ! output file name

  INTEGER,         PARAMETER :: nloc=1     ! number of source locations
  INTEGER,         PARAMETER :: nlvl=1     ! number of vertical grid levels
  INTEGER,         PARAMETER :: ntyp=1     ! number of pollutant grids
  INTEGER,         PARAMETER :: cpack=1    ! concentration packing on (off=0)

  INTEGER                    :: n          ! loop index

!-----------------------------------------------------

! concentration grid configuration
  CHARACTER(4) :: polid        ! pollutant identification
  INTEGER      :: conlvl       ! height m-AGL of grid plane
  REAL         :: clat         ! lower left corner concengration grid
  REAL         :: clon         ! lower left corner concentration grid
  REAL         :: dlat         ! grid spacing in degrees latitude
  REAL         :: dlon         ! grid spacing in degrees longitude
  INTEGER      :: nlon         ! number of longitude grid points
  INTEGER      :: nlat         ! number of latitude grid points
  INTEGER      :: yy           ! two digit year     

! WRF grid configuration
  CHARACTER(4) :: model_id     ! generating meteorological model
  REAL         :: gsize        ! base meteo grid resolution (km)
  REAL         :: blat         ! WRF grid lower left corner latitude
  REAL         :: blon         ! WRF grid lower left corner longitude
  REAL         :: tlat         ! WRF grid top right corner latitude
  REAL         :: tlon         ! WRF grid top right corner longitude

  INTEGER  :: nxp,nyp,nzp
  INTEGER  :: kzmax         ! model top (m)
  INTEGER  :: cgratio       ! number of concentration cells per WRF cell

  COMMON /gridpt/ nxp,nyp,nzp,kzmax,cgratio
  COMMON /conval/ polid,conlvl,clat,clon,dlat,dlon,nlat,nlon
  COMMON /wrfgrd/ model_id,gsize,blat,blon,tlat,tlon

!-----------------------------------------------------
! determine corner position and resolution of grid
! based upon the resolution of the WRF grid

  clat=NINT(blat)
  nlat=NINT(111.0*(NINT(tlat)-clat)/(gsize/cgratio))
  dlat=(tlat-clat)/nlat

! round to the nearest 0.05 deg
  dlat=0.05*NINT((dlat*100.0)/5.0)
  nlat=INT((tlat-clat)/dlat)

  clon=NINT(blon)
  dlon=dlat/COS(tlat/57.3)
  dlon=0.05*NINT((dlon*100.0)/5.0)
  nlon=NINT((tlon-clon)/dlon)

!-----------------------------------------------------
      
  OPEN (JUNIT,FILE=confile,FORM='UNFORMATTED',ACCESS='SEQUENTIAL')
  yy=datemet(1)-(datemet(1)/100)*100
  WRITE(JUNIT) model_id,yy,datemet(2:4),datemet(6),nloc,cpack 

! calculation start date and time for each source location
  yy=datecal(1)-(datecal(1)/100)*100
  DO n=1,nloc
     WRITE(JUNIT) yy,datecal(2:4),olat,olon,olvl,datecal(5)
  END DO

! horizontal grid index record
  WRITE(JUNIT) nlat,nlon,dlat,dlon,clat,clon

! vertical grid index record
  WRITE(JUNIT) nlvl, conlvl               !  (height,n=1,nlvl)

! pollutant identification record
  WRITE(JUNIT) ntyp, polid                !  (ident,n=1,ntyp)

END SUBROUTINE conset  
