! ----------------------------------------------------------------------------
! Maps the internal WRF concentration grid to the DATEM format variables   
! by summing the model calcuations to the previously loaded template
! for locations and sampling times. This routine is called at the end of
! a meteorological data period before the next time period is read. 
! ----------------------------------------------------------------------------
! Last Revised: 12 Aug 2020 
!               18 Nov 2020 - generalized time variable
!               29 Dec 2020 - simplified time match tests
! ----------------------------------------------------------------------------

SUBROUTINE DATSUM(datecal,cs)

  USE datval
  USE metval, ONLY: zt,xd,yd

  IMPLICIT NONE

  INTEGER(4), INTENT(IN) :: datecal(6) 
  REAL,       INTENT(IN) :: cs(:,:,:)   

  INTEGER(KIND=8)        :: csec,jsec,ksec,julsec 

  CHARACTER(4) :: polid        ! pollutant identification
  INTEGER      :: conlvl       ! height m-AGL of grid plane
  REAL         :: clat         ! lower left corner concengration grid
  REAL         :: clon         ! lower left corner concentration grid
  REAL         :: dlat         ! grid spacing in degrees latitude
  REAL         :: dlon         ! grid spacing in degrees longitude
  INTEGER      :: nlon         ! number of longitude grid points
  INTEGER      :: nlat         ! number of latitude grid points

  INTEGER      :: I,J,N,II,JJ,KK,KTOP ! local variables
  REAL         :: CTOT

  INTEGER  :: nxp,nyp,nzp,nxc,nyc 
  INTEGER  :: kzmax         ! model top (m)
  INTEGER  :: cgratio       ! number of concentration cells per WRF cell

  COMMON /gridpt/ nxp,nyp,nzp,kzmax,cgratio
  COMMON /conval/ polid,conlvl,clat,clon,dlat,dlon,nlat,nlon

!-----------------------------------------------------
! determine the depth of the concentration grid for output

  KK=2
  DO WHILE (KK.LE.NZP.AND.INT(ZT(KK)).LE.CONLVL)
     KK=KK+1      
  END DO
  KTOP=KK-1

! concentration grid limits
  nxc=1+cgratio*(nxp-1)
  nyc=1+cgratio*(nyp-1)

! convert current time to julian seconds
  CSEC=JULSEC(datecal(1),datecal(2),datecal(3),datecal(4),datecal(5),0)

  DO N=1,NREC

!    check limits if station outside of the WRF domain
     I=MAX(1,MIN(NXP,NINT(XPOS(N))))
     J=MAX(1,MIN(NYP,NINT(YPOS(N))))

!    sampling location concentration grid cell
     II=NINT(1.0+CGRATIO*(XPOS(N)-1.0))
     JJ=NINT(1.0+CGRATIO*(YPOS(N)-1.0))

!    again check limits if station outside of WRF domain
     II=MAX(1,MIN(NXC,II))
     JJ=MAX(1,MIN(NYC,JJ))

!    total mass in output layer
     CTOT=SUM(CS(II,JJ,2:KTOP))
!    vertically integrated cell volume
     CTOT=CTOT*CGRATIO*CGRATIO/(XD(I,J)*YD(I,J)*ZT(KTOP))

!    beginning and ending sample times
     JSEC=JULSEC(JYR(N),JMO(N),JDA(N),JHR(N),JMN(N),0)
     KSEC=JULSEC(KYR(N),KMO(N),KDA(N),KHR(N),KMN(N),0)

!    sample collection starts at current time
!    for this particular sampler, others still at -1.0
!    initializatiion occurs with each meteorology data read
     IF(CSEC.EQ.JSEC)THEN 
        CALC(n)=0.0
        CNTR(n)=1.0

!    sample collection ends at current time after advection
!    and corresponding with the end of the meteorological period
!    because CNTR starts at 1.0 it is not added at the end
     ELSEIF(CSEC.EQ.KSEC)THEN 
        CALC(n)=CALC(n)+CTOT
        CALC(n)=CALC(n)/CNTR(n)
        CNTR(n)=0.0

!    sampler collecting but not yet at sample end time
     ELSEIF(CNTR(N).GT.0.0)THEN
        CNTR(n)=CNTR(n)+1.0
        CALC(n)=CALC(n)+CTOT

!    sampler outside of collection interval
     ELSE
        CONTINUE
     END IF

  END DO

END SUBROUTINE datsum   
