!-------------------------------------------------------------------------------
! MEteorology CoNVert transforms the input meteorological data to variables or
! to units that can be used for the advection and dispersion calculations.
!-------------------------------------------------------------------------------
! Last Revised: 09 Dec 2019
!               06 Aug 2020 - add option to skip lower levels   
!               30 Oct 2020 - reversed sign of vertical motion ??? 
!               30 Dec 2020 - hydrostatic pressure option
!               04 Jan 2021 - corrected level skip section 
!               11 Jan 2021 - estimate tke data when missing
!               16 Jan 2021 - added conversion of RH to Q for pressure data
!-------------------------------------------------------------------------------

SUBROUTINE METCNV(tked,ii,jj,kt,tker,ptop,sg)

  USE metval

  IMPLICIT NONE

!-------------------------------
! argument list variables

  LOGICAL, INTENT(IN)    :: tked     ! tke field in meteorology
  INTEGER, INTENT(IN)    :: ii,jj    ! horizontal index
  INTEGER, INTENT(IN)    :: kt       ! number of active time
  REAL,    INTENT(IN)    :: tker     ! ratio of vert to horiz turb
  REAL,    INTENT(IN)    :: ptop     ! top pressure for hydrostatic
  REAL,    INTENT(IN)    :: sg(:)    ! hyddrostatic pressure levels 

!-------------------------------
! internal variables

  REAL,    PARAMETER :: grav  = 9.80616 ! gravity (m/s2)
  REAL,    PARAMETER :: rdry  = 287.04  ! dry air (J/Kg-K)
! REAL,    PARAMETER :: p2jm  = 100.0   ! mb to j/m3

! set kbot>2 to skip shallow levels near ground (e.g. 1=sfc 2=10m 3=25m, ...)
! when kbot=2 then the first level processed is actually the first data level AGL
  INTEGER            :: kbot            ! use as first input level above ground

  INTEGER            :: kk
  REAL               :: xl,zlvl,delp,delz,tvir,esat,mslp
  REAL               :: tratiov,tratioh,tkemin,tkemax,pblmin

  INTEGER            :: nxp,nyp,nzp
  INTEGER            :: kzmax       ! model top (m)
  INTEGER            :: cgratio     ! number of concentration cells to WRF cells

  COMMON /gridpt/ nxp,nyp,nzp,kzmax,cgratio
  COMMON /stblty/ tratiov,tratioh,tkemin,tkemax,pblmin

!--------------------------------

! insure mixed layer depth always exceeds minimum
  ZI(ii,jj,kt)=MAX(ZI(ii,jj,kt),PBLMIN)
 
  IF(PTOP.GE.0.0)THEN
!    hydrostatic models have no 3D pressure field 
!    use sigma profile and top pressure to define pressure profile     
!    ideally should use MSLP rather than 1013; not all data have field
     IF(sp(1,1,kt).GT.0.0)THEN
        MSLP=SP(ii,jj,kt)
     ELSE
        MSLP=1013.0
     END IF

     DO kk=1,nzp
        P(ii,jj,kk,kt)=SG(kk)*(MSLP-PTOP)+PTOP
     END DO
     KBOT=2 ! no not skip any input levels

  ELSE
!    for non-hydrostatic sigma level data  
!    check to insure pressure decreases with height (problems near the surface)
     DO kk=2,MIN(nzp,5)   
        IF(P(ii,jj,kk,kt).GE.P(ii,jj,kk-1,kt)) &    
           P(ii,jj,kk,kt)=0.5*(P(ii,jj,kk+1,kt)+P(ii,jj,kk-1,kt))
     END DO
     KBOT=3 ! skip the lowest sigma level (2); usually too close to surface
  END IF

! level two is the first one above the surface
! Note that HH(:,:,1) contains the terrain height (see inpsfc)

  IF(PTOP.GE.0.0)THEN
!    height field should be AGL for pressure level data
!    the input data height field is not used 
     ZLVL=-HH(ii,jj,1)
     HH(ii,jj,1)=ZLVL
  ELSE
     ZLVL=0.0
  END IF

  DO kk=2,nzp

!    relative humidity to specific (when Q>1)
     IF(Q(ii,jj,kk,kt).GT.0.1)THEN
        ESAT=EXP(21.4-(5351.0/T(ii,jj,kk,kt)))
        Q(ii,jj,kk,kt)=0.622*(Q(ii,jj,kk,kt)*ESAT/100.0)/P(ii,jj,kk,kt)
     END IF

!    hypsometric equation to integrate heights
     TVIR=T(ii,jj,kk,kt)*(1.0+0.61*Q(ii,jj,kk,kt))
     DELP=LOG(P(ii,jj,kk-1,kt)/P(ii,jj,kk,kt))
     DELZ=DELP*RDRY*TVIR/GRAV
     ZLVL=ZLVL+DELZ
     HH(ii,jj,kk)=ZLVL

!    estimate from mixing depth if missing
     IF(.NOT.tked)THEN
        IF(ZLVL.LE.ZI(ii,jj,kt))THEN
           E(ii,jj,kk,kt)=0.001*ZI(ii,jj,kt)
        ELSE
           E(ii,jj,kk,kt)=TKEMIN
        END IF
     END IF
 
!    assume minimum and maximum turbulence values       
     E(ii,jj,kk,kt) = MAX(MIN(E(ii,jj,kk,kt),TKEMAX),TKEMIN)

!##  Set the TKE constant for testing purposes      
!##  Sw(m/s)   W'W'(m2/s2)   TKE      2Sw after 1200 s
!##  0.552     0.305         1.0      1324 m
!##  0.247     0.061         0.2       593 m
!##  0.0416    0.001735      0.0056    100 m
!##  E(ii,jj,kz,kt)=0.0056      

!    From the definition TKE=0.5(U'2+V'2+W'2) and the partition ratio 
!    defined by TKER=W'2/(U'2+V'2), compute W'2
     XL=1.0/TKER
     WW(ii,jj,kk,kt)=2.0*E(ii,jj,kk,kt)/(1.0+XL)

!    simple equal partition between u'2 and v'2 
     UU(ii,jj,kk,kt)=SQRT(0.50*XL*WW(ii,jj,kk,kt))
     VV(ii,jj,kk,kt)=UU(ii,jj,kk,kt)         

!##  simple equal partition between u'2 and v'2 can be replaced if we
!##  define the turbulence vectors to be in the same ratio as the velocity 
!##  vectors, such that for the the U component: U'2/(U'2+V'2)=U2/(U2+V2),
!##  and therefore U'2=U2*(U'2+V'2)/(U2+V2) and (U'2+V'2)=W'2/TKER=W'2*XL
!##  REAL               :: usqr,vsqr,uvsq,uuvv
!##  USQR=U(ii,jj,kk,kt)*U(ii,jj,kk,kt)
!##  VSQR=V(ii,jj,kk,kt)*V(ii,jj,kk,kt)
!##  UVSQ=MAX(USQR+VSQR,0.0001)
!##  UUVV=WW(ii,jj,kk,kt)*XL/UVSQ    ! (U'2+V'2)/(U2+V2)   
!##  UU(ii,jj,kk,kt)=SQRT(USQR*UUVV) ! u'2
!##  VV(ii,jj,kk,kt)=SQRT(VSQR*UUVV) ! v'2

!    convert from variance to sigma
     WW(ii,jj,kk,kt)=SQRT(WW(ii,jj,kk,kt))

!    convert vertical velocity from mb/s to m/s 
!   (mb/s) x (m/mb) 
!    WVEL  x DELZ/(P2-P1) 
     W(ii,jj,kk,kt)=W(ii,jj,kk,kt)*DELZ/(P(ii,jj,kk,kt)-P(ii,jj,kk-1,kt))

!    fill in missing values at the surface (really 2/10 meters)
!    regardless of whether level #2 is skipped 
     IF(kk.EQ.2)THEN
         W(ii,jj,1,kt)= W(ii,jj,kk,kt)
         Q(ii,jj,1,kt)= Q(ii,jj,kk,kt)
         E(ii,jj,1,kt)= E(ii,jj,kk,kt)
        UU(ii,jj,1,kt)=UU(ii,jj,kk,kt)
        VV(ii,jj,1,kt)=VV(ii,jj,kk,kt)
        WW(ii,jj,1,kt)=WW(ii,jj,kk,kt)
     END IF

!##  force winds for testing 5.14 m/s = 1 deg lat / 6 hours
!##  U(ii,jj,kk,kt)=0.0
!##  V(ii,jj,kk,kt)=5.14
!##  W(ii,jj,kk,kt)=0.0

  END DO

! fill in levels below KTOP with KTOP when KTOP>2 
  IF(kbot.GT.2)THEN
     DO kk=2,(kbot-1)
        U(ii,jj,kk,kt)=U(ii,jj,kbot,kt)
        V(ii,jj,kk,kt)=V(ii,jj,kbot,kt)
        W(ii,jj,kk,kt)=W(ii,jj,kbot,kt)
        T(ii,jj,kk,kt)=T(ii,jj,kbot,kt)
        Q(ii,jj,kk,kt)=Q(ii,jj,kbot,kt)
        E(ii,jj,kk,kt)=E(ii,jj,kbot,kt)
     END DO 
  END IF

END SUBROUTINE metcnv
