!---------------------------------------------------------------
! Defines the vertical grid structure by computing the average
! height over the domain for each level and also defines the
! the relationship between vertical index and height. This
! routine can be called once at the beginning or after each 
! new meteorological data input.
!---------------------------------------------------------------
! Last Revised: 12 Nov 2020
!               16 Jan 2021 - added option to select a point not average
!---------------------------------------------------------------

SUBROUTINE METGRD(ii,jj)

  USE metval, ONLY: zd,zt,zk,hh

  IMPLICIT none
 
  INTEGER, INTENT(IN) :: ii,jj ! option to use local rather than average

  INTEGER  :: nz,kk            ! indicies
  INTEGER  :: nxp              ! number of horizontal W->E grid points
  INTEGER  :: nyp              ! number of horizontal S->N grid points
  INTEGER  :: nzp              ! number of vertical levels (including sfc)
  INTEGER  :: kzmax            ! model top in meters, must exceed data top
  INTEGER  :: cgratio          ! numb of concentration grid cells to WRF cells

  COMMON /gridpt/ nxp,nyp,nzp,kzmax,cgratio

  IF(ii.GT.0.AND.jj.GT.0)THEN
     ZT(1)=HH(ii,jj,1)
  ELSE
     ZT(1)=0.0         
  END IF
  ZD(1)=0.0

  DO nz=2,nzp
!    average height over the domain for each level
!    note that HH(:,:,1) contains the terrain height while 
!    all the other levels are the height above ground-level

     IF(ii.GT.0.AND.jj.GT.0)THEN
!       when position is set (<>0) then the relationship
!       between height and index only applies at that location;
!       an option enabled for pressure level input data
        ZT(nz)=HH(ii,jj,nz)

     ELSE
        ZT(nz)=SUM(HH(:,:,nz))/FLOAT(nxp*nyp)
     END IF

!    height difference between levels
     ZD(nz)=ZT(nz)-ZT(nz-1)
  END DO

! check compiled vertical dimension
  IF(INT(zt(nzp)).GT.kzmax)THEN
     WRITE(*,*)'Meteorology top level = ',INT(zt(nzp))
     WRITE(*,*)'Exceeds compiled top  = ',kzmax
     WRITE(*,*)'Change KZMAX and recompile!'
     STOP 900
  END IF
 
! defines the relationship between vertical index and height
! where ZK contains the fractional index value by meter AGL and
! where an index=1.0 is the ground surface and index=2 is the second
! meteorological data level above the surface
! K  ~ m-AGL
! ...   ...
!   3    25
!   2     9
!   1     0
! For example, heights from 1 to 8 m-AGL would have ZK 
! values of about 1.1, 1.2, 1.3, etc, for each increasing meter

  DO nz=2,nzp   
  DO kk=INT(ZT(nz-1)),INT(ZT(nz))
     IF(kk.GT.0.AND.kk.LE.kzmax) &  
        ZK(kk)=FLOAT(nz-1)+(FLOAT(kk)-ZT(nz-1))/ZD(nz)    
  END DO      
  END DO
  ZK(INT(ZT(nzp)):)=FLOAT(nzp)

END SUBROUTINE metgrd
