!-------------------------------------------------------------------------------
! Routine to read and unpack all the upper fields. The level indicator in the
! LABEL field would show a value of one or more. However, the internal data 
! array would have an index value of 1 at the surface.
!-------------------------------------------------------------------------------
! Last Revised: 09 Dec 2019
!               08 Aug 2020 - updated common block
!               19 Nov 2020 - time averaged WRF mass velocities
!               16 Jan 2021 - also read RH into moisture field when no Q
!-------------------------------------------------------------------------------

SUBROUTINE METLVL(KUNIT,KT,KREC,NVLVL,TAVG)

  USE metval

  IMPLICIT NONE

!-------------------------------
! argument list variables

  INTEGER,      INTENT(IN)    :: kunit     ! input device unit number
  INTEGER,      INTENT(IN)    :: kt        ! number of active time
  INTEGER,      INTENT(IN)    :: krec      ! record number of index record
  INTEGER,      INTENT(IN)    :: nvlvl     ! number of variables per level
  LOGICAL,      INTENT(IN)    :: tavg      ! flag for time averaged data


! one and two dimensional dummy variables for I/O routines
  CHARACTER(1), ALLOCATABLE   :: cdata(:)
  REAL,         ALLOCATABLE   :: rvar(:,:), diff(:,:)

  SAVE cdata,rvar,diff

!-------------------------------
! internal variables

  CHARACTER(50)               :: label 
  CHARACTER(4)                :: varb
  INTEGER                     :: nv,jrec,kret
  INTEGER                     :: iy,im,id,ih,ic,ll,nexp
  REAL                        :: prec,var1

  INTEGER                     :: nxp,nyp,nzp
  INTEGER                     :: kzmax         ! model top (m)
  INTEGER                     :: cgratio       ! concentration cells to WRF cells

!-------------------------------
  INTERFACE
  SUBROUTINE PAKINP(RVAR,CVAR,PREC,NEXP,VAR1)
  IMPLICIT NONE
  REAL,          INTENT(OUT)   :: rvar (:,:)     ! real data unpacked
  CHARACTER(1),  INTENT(IN)    :: cvar (:)       ! packed input of NX*NY
  REAL,          INTENT(IN)    :: prec           ! precision of packed data 
  INTEGER,       INTENT(IN)    :: nexp           ! packing scaling exponent
  REAL,          INTENT(IN)    :: var1           ! value of array at (1,1)
  END SUBROUTINE pakinp
  END INTERFACE
!--------------------------------

  COMMON /gridpt/ nxp,nyp,nzp,kzmax,cgratio

!--------------------------------

  IF(.NOT.ALLOCATED(cdata))THEN
     ALLOCATE (cdata(nxp*nyp),STAT=kret)
     ALLOCATE (rvar(nxp,nyp),diff(nxp,nyp),STAT=kret)
  END IF

  JREC=KREC
  DO NV=1,NVLVL
     READ(KUNIT,REC=JREC,ERR=910)LABEL,CDATA
     READ(LABEL,'(6I2,2X,A4,I4,2E14.7)') IY,IM,ID,IH,IC,LL,VARB,NEXP,PREC,VAR1
     IF(LL.LE.0) GOTO 920
     LL=LL+1  ! data array index=1 is the surface level (i.e. LL=0)

     SELECT CASE (VARB)
        CASE ('UWND') 
           CALL PAKINP(U(:,:,LL,KT),CDATA,PREC,NEXP,VAR1)
           IF(tavg) U(:,:,LL,KT)=U(:,:,LL,KT)*MSFU(:,:)/MUU0(:,:)
        CASE ('VWND') 
           CALL PAKINP(V(:,:,LL,KT),CDATA,PREC,NEXP,VAR1)
           IF(tavg) V(:,:,LL,KT)=V(:,:,LL,KT)*MSFV(:,:)/MUV0(:,:)
        CASE ('WWND')
           CALL PAKINP(W(:,:,LL,KT),CDATA,PREC,NEXP,VAR1)
           IF(tavg) W(:,:,LL,KT)=W(:,:,LL,KT)*MSFT(:,:)/100.0
        CASE ('DIFW')
!          diff=(orginal_value)-(packed_value)
           CALL PAKINP(DIFF,CDATA,PREC,NEXP,VAR1)
           IF(tavg) DIFF(:,:)=DIFF(:,:)*MSFT(:,:)/100.0
           RVAR=DIFF+W(:,:,LL,KT)
           W(:,:,LL,KT)=RVAR 
        CASE ('TEMP')
           CALL PAKINP(T(:,:,LL,KT),CDATA,PREC,NEXP,VAR1)
        CASE ('THET')
           CALL PAKINP(T(:,:,LL,KT),CDATA,PREC,NEXP,VAR1)
           T(:,:,LL,KT)=T(:,:,LL,KT)*(P(:,:,LL,KT)/1000.0)**0.286
        CASE ('SPHU','RELH') 
           CALL PAKINP(Q(:,:,LL,KT),CDATA,PREC,NEXP,VAR1)
        CASE ('PRES') 
           CALL PAKINP(P(:,:,LL,KT),CDATA,PREC,NEXP,VAR1)
        CASE ('TKEN')
           CALL PAKINP(E(:,:,LL,KT),CDATA,PREC,NEXP,VAR1)
        CASE ('DIFT')
!          diff=(orginal_value)-(packed_value)
           CALL PAKINP(DIFF,CDATA,PREC,NEXP,VAR1)
           RVAR=DIFF+E(:,:,LL,KT)
           E(:,:,LL,KT)=RVAR
     END SELECT
     JREC=JREC+1

  END DO
  RETURN 

!----------------------------
! error termination messages

  910 WRITE(*,*)'*ERROR* metlvl: reading data record in meteo file'
      WRITE(*,*)' record number: ',JREC
      STOP 900
  920 WRITE(*,*)'*ERROR* metlvl: level field less than or equal to zero'
      WRITE(*,*)' record number: ',JREC
      WRITE(*,*)' label: ',LABEL
      STOP 900

END SUBROUTINE metlvl
