!--------------------------------------------------------------------
! Open the meteorological data file and set all the grid and file
! positioning variables for conformal grid WRF only. The ADAPT code
! uses the CMAPF routines (version 1.0) to transform between earth
! and model grid coordinates. Version 1.0 covers the standard conformal 
! map projections centered at the North and South Pole, namely the
! Polar Stereographic, the Mercator, and the Lambert Conformal projections.
! See https://www.arl.noaa.gov/hysplit/cmapf-mapping-routines/
! Also version 1.0 is described more fully in an article, Conformal Map 
! Transformations for Meteorological Modelers, by Albion D. Taylor,
! published in the February, 1997 issue of Computers and Geosciences
! (v23, no 1). Version 1.0 is available for download from 
! https://www.arl.noaa.gov/wp_arl/wp-content/uploads/utilities/cmap/
! cmapf.v1_0.tar.gz
! or from the Computers and Geosciences site
! https://www.iamg.org/documents/oldftp/VOL23/v23-1-5.tar.Z
!--------------------------------------------------------------------
! Last Revised: 11 Nov 2020
!--------------------------------------------------------------------

SUBROUTINE METPRJ  (kunit,diag,gbase,datemet)

  USE metval, ONLY: xd,yd,glat,glon

  IMPLICIT none

  INTEGER, INTENT(IN)  :: kunit      ! meteorological unit number
  LOGICAL, INTENT(IN)  :: diag       ! diagnostic output
  REAL,    INTENT(OUT) :: gbase(9)   ! array that defines grid conversions
  INTEGER, INTENT(OUT) :: datemet(6) ! start year and month

  CHARACTER(50)  :: LABEL
  CHARACTER(108) :: HEADER

! conformal map projection variables for meteorological grid
  CHARACTER(4)   :: model_id    ! model data identification
  INTEGER        :: nxp,nyp,nzp ! number of grid points
  INTEGER        :: ii,jj       ! loop indicies
  INTEGER        :: icm,icx     ! minutes and extended forecast hr 
  REAL           :: pole_lat    ! pole latitude of projection
  REAL           :: pole_lon    ! pole longitude of projection
  REAL           :: ref_lat     ! reference latitude (size valid)
  REAL           :: ref_lon     ! reference longitude
  REAL           :: gsize       ! grid size at ref lat lon
  REAL           :: orient      ! orientation
  REAL           :: tang_lat    ! tangent latitude
  REAL           :: sync_xp     ! synchronization point in x
  REAL           :: sync_yp     ! synchronization point in y
  REAL           :: sync_lat    ! latitude of sync point
  REAL           :: sync_lon    ! longitude of sync point
  REAL           :: dummy       ! reserved for future use
  REAL           :: eqvlat      ! equivalent latitude function
  REAL           :: cgszxy      ! grid size function
  REAL           :: txp,typ     ! tangent x and y point

  REAL           :: blat,blon   ! bottom lower left corner
  REAL           :: tlat,tlon   ! top upper right corner

  INTEGER        :: kzmax       ! model top (m)
  INTEGER        :: cgratio     ! number of concentration cells per WRF 

  COMMON /wrfgrd/ model_id,gsize,blat,blon,tlat,tlon
  COMMON /gridpt/ nxp,nyp,nzp,kzmax,cgratio

! decode the extended portion of the index record
  READ(KUNIT,REC=1)LABEL,HEADER(1:108)
  READ(LABEL,'(5I2)') datemet(1:4),datemet(6)
  READ(HEADER(1:108),'(A4,I3,I2,12F7.0)')                               &
       MODEL_ID,ICX,ICM,POLE_LAT,POLE_LON,REF_LAT,REF_LON,GSIZE,ORIENT, &  
       TANG_LAT,SYNC_XP,SYNC_YP,SYNC_LAT,SYNC_LON,DUMMY

  datemet(5)=icm                ! minutes field
  IF(icx.GT.99) datemet(6)=icx  ! extended forecast hours (3-digits)

  IF(REF_LAT.EQ.-999.99)THEN
!    when AER-WRF grids are defined by two tangent latitudes, it is indicated 
!    by special mapping definitions defined by a missing field in reference 
!    latitude, then use the EQVLAT function to convert to a single tangent 
!    latitude (for STLMBR)     
     TANG_LAT=EQVLAT(TANG_LAT,DUMMY)
     ORIENT=0.0
     REF_LAT=TANG_LAT
     DUMMY=0.0
  END IF

! initialize grid conversion variable (into gbase)
  CALL STLMBR(GBASE,TANG_LAT,REF_LON)

! use single point grid definition
  CALL STCM1P(GBASE,SYNC_XP,SYNC_YP,SYNC_LAT,SYNC_LON,REF_LAT,REF_LON, &
              GSIZE,ORIENT)

! save max and min points to common block
  blat=90.0      
  blon=180.0       
  tlat=-90.0        
  tlon=-180.0        

! define horizontal grid spacing
  DO II=1,NXP
  DO JJ=1,NYP
     CALL CXY2LL(GBASE,FLOAT(II),FLOAT(JJ),GLAT(II,JJ),GLON(II,JJ))
!    meters per grid cell
     XD(II,JJ)=CGSZXY(GBASE,FLOAT(II),FLOAT(JJ))*1000.0
!    conformal projection
     YD(II,JJ)=XD(II,JJ)

     blat=MIN(blat,glat(ii,jj))
     blon=MIN(blon,glon(ii,jj))
     tlat=MAX(tlat,glat(ii,jj))
     tlon=MAX(tlon,glon(ii,jj))
  END DO
  END DO

  IF(diag)THEN
     WRITE(*,*)'From METPRJ .......'
     WRITE(*,*)'Pole point lat/lon:',pole_lat,pole_lon
     WRITE(*,*)' Reference lat/lon:',ref_lat,ref_lon
     WRITE(*,*)'    Grid size (km):',gsize
     WRITE(*,*)'       Orientation:',orient 
     WRITE(*,*)'   Tang lat / Cone:',tang_lat
     WRITE(*,*)'      Sync pnt x,y:',sync_xp,sync_yp
     WRITE(*,*)'  Sync pnt lat,lon:',sync_lat,sync_lon
     WRITE(*,*)'Grid spacing midpt: ',XD(NXP/2,NYP/2)
     WRITE(*,*)'Lower left  corner: ',blat,blon
     WRITE(*,*)'Upper right corner: ',tlat,tlon
     CALL CLL2XY(GBASE,TANG_LAT,0.0,TXP,TYP)
     WRITE(*,*)' Tangent point x,y:',TXP,TYP
     WRITE(*,*)  
  END IF

END SUBROUTINE metprj
