!--------------------------------------------------------------------
! Open the meteorological data file and set all the grid variables 
! for positioning 
!--------------------------------------------------------------------
! Last Revised: 11 Nov 2020
!               25 Nov 2020 - dimension for multiple meteo files
!               02 Dec 2020 - TKE not needed for trajectories
!               30 Dec 2020 - saved vertical sigma coordinate
!               01 Jan 2021 - added test for HRRR
!               11 Jan 2021 - bring TKE data exist to main
!--------------------------------------------------------------------

SUBROUTINE METSET (kunit,metfile,traj,tavg,tked,nxp,nyp,nzp,nvsfc,nvlvl, &
                   numper,numndx,delm,ptop,sg)   

  IMPLICIT none

  CHARACTER(256), INTENT(IN) :: metfile(:)
  INTEGER, INTENT(IN)        :: kunit
  LOGICAL, INTENT(IN)        :: traj         ! flag for trajectories
  LOGICAL, INTENT(OUT)       :: tavg         ! time averaged meteorololgy
  LOGICAL, INTENT(OUT)       :: tked         ! tke data exists in meteorology 
  INTEGER, INTENT(OUT)       :: nxp,nyp,nzp  ! meteo grid dimensions
  INTEGER, INTENT(OUT)       :: nvsfc,nvlvl  ! numb variables at sfc, aloft
  INTEGER, INTENT(OUT)       :: numper       ! numb records per time
  INTEGER, INTENT(OUT)       :: numndx       ! numb of index records 
  INTEGER, INTENT(OUT)       :: delm         ! time between meteorology (s)
  REAL,    INTENT(OUT)       :: ptop         ! pressure top for hydrostatic
  REAL,    INTENT(OUT)       :: sg(:)        ! hydrostatic sigma levels      

  INTEGER                    :: ldat         ! data length (nxp*nyp)
  INTEGER                    :: lrec         ! meteorological data record length

! meteorological input file grid variables
  CHARACTER(2)    :: CGRID 
  CHARACTER(4)    :: KVAR,MODEL_ID
  CHARACTER(50)   :: LABEL
  CHARACTER(5000) :: HEADER
  INTEGER         :: Z_FLAG,LENH
  REAL            :: PBOT

  INTEGER         :: iyr,imo,ida,ihr,imn,ifh
  INTEGER         :: jyr,jmo,jda,jhr,jmn,jfh
  INTEGER         :: k,l,knx,kny,kol,numvar,chksum,nrec
  CHARACTER(4)    :: varbid
  LOGICAL         :: vpres,ftest

! check to get a clearer error message if not found
  INQUIRE(FILE=metfile(1),EXIST=FTEST)
  IF(.NOT.FTEST)THEN
     WRITE(*,*)'File not found: ',TRIM(metfile(1))
     STOP
  END IF

! open file to decode the standard label (50) plus the 
! fixed portion (108) of the extended header
  OPEN(KUNIT,FILE=METFILE(1),STATUS='OLD',ACTION='READ',RECL=158,  &
       ACCESS='DIRECT',FORM='UNFORMATTED')

! decode the standard portion of the index record
  READ(KUNIT,REC=1)LABEL,HEADER(1:108)
  READ(LABEL,'(5I2,2X,A2,A4)') IYR,IMO,IDA,IHR,IFH,CGRID,KVAR
  IF(KVAR.NE.'INDX')THEN
     WRITE(*,*)'Meteorology file error: ', LABEL
     STOP
  END IF

! decode the extended portion of the index record
  READ(HEADER(1:108),'(A4,82X,F7.0,3I3,I2,I4)')    &
       MODEL_ID,PTOP,NXP,NYP,NZP,Z_FLAG,LENH
  READ(HEADER(8:9),'(I2)')IMN ! minutes field from extended
  IF(LENH.GT.5000)THEN
    WRITE(*,*)'Meteorological data header: ',LENH
    WRITE(*,*)'Exceeds compiled limit of 5000'
    STOP
  END IF

! grid id variable needed if either dimension is beyond 3 digits
  KNX=ICHAR(CGRID(1:1))
  KNY=ICHAR(CGRID(2:2))
  IF(KNX.GE.64.OR.KNY.GE.64)THEN
     NXP=(KNX-64)*1000+NXP
     NYP=(KNY-64)*1000+NYP
  END IF

! meteorology file data structure
  LDAT=NXP*NYP        ! length of data part of record
  LREC=LDAT+50        ! data part plus header
  NUMNDX=LENH/LDAT+1  ! number of index records
  IF(NUMNDX.GT.1)THEN
     WRITE(*,*)'More than one index record not permitted: ',NUMNDX
     STOP
  END IF

! close file and reopen with proper length
  CLOSE (KUNIT)
  OPEN(KUNIT,FILE=METFILE(1),STATUS='OLD',ACTION='READ',RECL=LREC,  &
       ACCESS='DIRECT',FORM='UNFORMATTED')
  READ(KUNIT,REC=1)LABEL,HEADER(1:LENH)

  tked=.false.
  tavg=.false.
  vpres=.false.

! determine the number of surface and upper level variables
  KOL=109
  NREC=NUMNDX
  DO L=1,NZP
     READ(HEADER(KOL:KOL+7),'(F6.2,I2)') SG(L),NUMVAR
     KOL=KOL+8
     DO K=1,NUMVAR
        READ(HEADER(KOL:KOL+7),'(A4,I3)') VARBID,CHKSUM 
        KOL=KOL+8
        NREC=NREC+1  ! one data record per variable and level
        IF(VARBID.EQ.'TKEN') tked=.true.
        IF(VARBID.EQ.'MUU0') tavg=.true.
        IF(VARBID.EQ.'PRES') vpres=.true.
     END DO
     IF(L.EQ.1)nvsfc=numvar
     IF(L.EQ.2)nvlvl=numvar
  END DO

! TKE field is estimated when not available
! Mixing not computed for trajectories
  IF(.NOT.tked)THEN
     IF(.NOT.traj) &   
        WRITE(*,*)'TKE field missing in meteorological data file!'
  END IF

! A missing vertical pressure field means that the model is hydrostatic
! and the pressure profile must be built from the sigma level data
! using the value of the pressure (PTOP) at the top of the model.
! For non-hydrostatic models the PTOP field is not needed.
  IF(vpres)PTOP=-1.0

  IF(Z_FLAG.NE.1)THEN
     IF(.NOT.vpres)THEN
        WRITE(*,*)'Upper level pressures are missing'
        WRITE(*,*)'Vertical coordinate not sigma: ',Z_FLAG
        WRITE(*,*)'... From meteorological model: ',MODEL_ID  

!       pressure level data; convert pressures to sigma
        IF(Z_FLAG.EQ.2)THEN
           SG(1)=1.0
           PBOT=1013.0 
           PTOP=SG(NZP)
           DO L=2,NZP
              SG(L)=(SG(L)-PTOP)/(PBOT-PTOP)
           END DO
        END IF
     END IF
  END IF

! skip to the next time period to determine the number of records
  KVAR='    '
  NREC=NUMNDX+1
  DO WHILE (KVAR.NE.'INDX')
     READ(KUNIT,REC=NREC)LABEL,HEADER(1:9) 
     READ(LABEL,'(5I2,2X,A2,A4)')JYR,JMO,JDA,JHR,JFH,CGRID,KVAR
     NREC=NREC+1
  END DO
  NUMPER=NREC-2  ! number of records per time period

! simple time difference assuming year,month,day are the same
  READ(HEADER(8:9),'(I2)')JMN  ! minutes field from extended
  DELM=(JHR-IHR)*3600 + (JMN-IMN)*60
  
END SUBROUTINE metset
