!-------------------------------------------------------------------------------
! Routine to read and unpack all the surface fields identified by a level number
! of zero in the LABEL field. The data are place into the internal array at
! an index of one, which always corresponds to the surface 
!-------------------------------------------------------------------------------
! Last Revised: 09 Dec 2019
!               09 Aug 2020 - changed height variable from ZT to HH
!               19 Nov 2020 - added WRF velocity flux variables
!               17 Jan 2021 - added mean sea level pressure: sp
!-------------------------------------------------------------------------------

SUBROUTINE METSFC(KUNIT,KT,KREC,NVSFC)

  USE metval

  IMPLICIT NONE

!-------------------------------
! argument list variables

  INTEGER,      INTENT(IN)    :: kunit     ! input device unit number
  INTEGER,      INTENT(IN)    :: kt        ! number of active time
  INTEGER,      INTENT(IN)    :: krec      ! record number of index record
  INTEGER,      INTENT(IN)    :: nvsfc     ! number of variables per level

! one and two dimensional dummy variables for I/O routines
  CHARACTER(1), ALLOCATABLE   :: cdata(:)
  REAL,         ALLOCATABLE   :: rvar(:,:), diff(:,:)

!-------------------------------
! internal variables

  CHARACTER(50)               :: label 
  CHARACTER(4)                :: varb
  INTEGER                     :: nv,jrec,kret
  INTEGER                     :: iy,im,id,ih,ic,ll,nexp
  REAL                        :: prec,var1

  INTEGER                     :: nxp,nyp,nzp
  INTEGER                     :: kzmax         ! model top (m)
  INTEGER                     :: cgratio       ! concentration cells to WRF cells

  SAVE cdata,rvar,diff

!-------------------------------
  INTERFACE
  SUBROUTINE PAKINP(RVAR,CVAR,PREC,NEXP,VAR1)
  IMPLICIT NONE
  REAL,          INTENT(OUT)   :: rvar (:,:)     ! real data unpacked
  CHARACTER(1),  INTENT(IN)    :: cvar (:)       ! packed input of NX*NY
  REAL,          INTENT(IN)    :: prec           ! precision of packed data 
  INTEGER,       INTENT(IN)    :: nexp           ! packing scaling exponent
  REAL,          INTENT(IN)    :: var1           ! value of array at (1,1)
  END SUBROUTINE pakinp
  END INTERFACE
!--------------------------------

  COMMON /gridpt/ nxp,nyp,nzp,kzmax,cgratio

!--------------------------------

  IF(.NOT.ALLOCATED(cdata))THEN
     ALLOCATE (cdata(nxp*nyp),STAT=kret)
     ALLOCATE (rvar(nxp,nyp),diff(nxp,nyp),STAT=kret)
  END IF

! flag variables at 1,1 for missing
  SP(1,1,KT)=-1.0

  JREC=KREC
  DO NV=1,NVSFC
     READ(KUNIT,REC=JREC,ERR=910)LABEL,CDATA
     READ(LABEL,'(6I2,2X,A4,I4,2E14.7)') IY,IM,ID,IH,IC,LL,VARB,NEXP,PREC,VAR1
     IF(LL.NE.0) GOTO 920

     SELECT CASE (VARB)
        CASE ('PRSS')
           CALL PAKINP(P(:,:,1,KT),CDATA,PREC,NEXP,VAR1)
        CASE ('T02M') 
           CALL PAKINP(T(:,:,1,KT),CDATA,PREC,NEXP,VAR1)
        CASE ('U10M')
           CALL PAKINP(U(:,:,1,KT),CDATA,PREC,NEXP,VAR1)
        CASE ('V10M')
           CALL PAKINP(V(:,:,1,KT),CDATA,PREC,NEXP,VAR1)
        CASE ('SHGT')
           CALL PAKINP(HH(:,:,1),CDATA,PREC,NEXP,VAR1)
        CASE ('PBLH')
           CALL PAKINP(ZI(:,:,KT),CDATA,PREC,NEXP,VAR1)
        CASE ('TPP1')
           CALL PAKINP(RT(:,:,KT),CDATA,PREC,NEXP,VAR1)
        CASE ('DIFR')
!          diff=(orginal_value)-(packed_value)
           CALL PAKINP(DIFF,CDATA,PREC,NEXP,VAR1)
           RVAR=DIFF+RT(:,:,KT)             
           RT(:,:,KT)=RVAR
        CASE ('USTR')
           CALL PAKINP(UF(:,:,KT),CDATA,PREC,NEXP,VAR1)
        CASE ('SHTF')
           CALL PAKINP(HF(:,:,KT),CDATA,PREC,NEXP,VAR1)
        CASE ('DSWF')
           CALL PAKINP(DS(:,:,KT),CDATA,PREC,NEXP,VAR1)
        CASE ('MSLP')
           CALL PAKINP(SP(:,:,KT),CDATA,PREC,NEXP,VAR1)

        CASE ('MUU0')
           CALL PAKINP(MUU0(:,:),CDATA,PREC,NEXP,VAR1)
        CASE ('MUV0')
           CALL PAKINP(MUV0(:,:),CDATA,PREC,NEXP,VAR1)
        CASE ('MSFU')
           CALL PAKINP(MSFU(:,:),CDATA,PREC,NEXP,VAR1)
        CASE ('MSFV')
           CALL PAKINP(MSFV(:,:),CDATA,PREC,NEXP,VAR1)
        CASE ('MSFT')
           CALL PAKINP(MSFT(:,:),CDATA,PREC,NEXP,VAR1)
     END SELECT
     JREC=JREC+1
  END DO
  RETURN 

!----------------------------
! error termination messages

  910 WRITE(*,*)'*ERROR* metsfc: reading data record in meteo file'
      WRITE(*,*)' record number: ',JREC
      STOP 900
  920 WRITE(*,*)'*ERROR* metsfc: level field not equal to zero'
      WRITE(*,*)' record number: ',JREC
      WRITE(*,*)' label: ',LABEL
      STOP 900

END SUBROUTINE metsfc
