!###############################################################################
! NORMAL, a FORTRAN90 code which returns a sequence of normally distributed 
! pseudorandom numbers. For more information see -
! https://people.sc.fsu.edu/~jburkardt/f_src/normal/normal.html
! 
! NORMAL is based on two simple ideas:
!
! the use of a fairly simple uniform pseudorandom number generator, which can
! be implemented in software; the use of the Box-Muller transformation to 
! convert pairs of uniformly distributed random values to pairs of normally
! distributed random values. Using these ideas, it is not too hard to generate 
! normal sequences of real or complex values, of single or double precision.
! These values can be generated as single quantities, vectors or matrices. An
! associated seed actually determines the sequence. Varying the seed will 
! result in producing a different sequence.
! 
! The fundamental underlying random number generator used here is based on a 
! simple, old, and limited linear congruential random number generator 
! originally used in the IBM System 360.
!
! Licensing:
! The computer code and data files described and made available on this web
! page are distributed under the GNU LGPL license.
! https://www.gnu.org/licenses/lgpl-3.0.en.html

!*******************************************************************************
!
!  R4_NORMAL_01 returns a unit pseudonormal R4.
!
!  Discussion:
!
!    The standard normal probability distribution function (PDF) has
!    mean 0 and standard deviation 1.
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license.
!
!  Modified:
!
!    06 August 2013
!
!  Author:
!
!    John Burkardt
!
!  Parameters:
!
!    Input/output, integer ( kind = 4 ) SEED, a seed for the random
!    number generator.
!
!    Output, real ( kind = 4 ) R4_NORMAL_01, a sample of the standard
!    normal PDF.
!

function r4_normal_01 ( seed )

  implicit none

  real ( kind = 4 ) r1
  real ( kind = 4 ) r2
  real ( kind = 4 ) r4_normal_01
  real ( kind = 4 ), parameter :: r4_pi = 3.141592653589793E+00
  real ( kind = 4 ) r4_uniform_01
  integer ( kind = 4 ) seed
  real ( kind = 4 ) x

  r1 = r4_uniform_01 ( seed )
  r2 = r4_uniform_01 ( seed )
  x = sqrt ( - 2.0E+00 * log ( r1 ) ) * cos ( 2.0E+00 * r4_pi * r2 )

  r4_normal_01 = x

  return
end

!*****************************************************************************80
!
!! R4_UNIFORM_01 returns a unit pseudorandom R4.
!
!  Discussion:
!
!    An R4 is a real ( kind = 4 ) value.
!
!    This routine implements the recursion
!
!      seed = 16807 * seed mod ( 2^31 - 1 )
!      r4_uniform_01 = seed / ( 2^31 - 1 )
!
!    The integer arithmetic never requires more than 32 bits,
!    including a sign bit.
!
!    If the initial seed is 12345, then the first three computations are
!
!      Input     Output      R4_UNIFORM_01
!      SEED      SEED
!
!         12345   207482415  0.096616
!     207482415  1790989824  0.833995
!    1790989824  2035175616  0.947702
!
!  Licensing:
!
!    This code is distributed under the GNU LGPL license.
!
!  Modified:
!
!    31 May 2007
!
!  Author:
!
!    John Burkardt
!
!  Reference:
!
!    Paul Bratley, Bennett Fox, Linus Schrage,
!    A Guide to Simulation,
!    Second Edition,
!    Springer, 1987,
!    ISBN: 0387964673,
!    LC: QA76.9.C65.B73.
!
!    Bennett Fox,
!    Algorithm 647:
!    Implementation and Relative Efficiency of Quasirandom
!    Sequence Generators,
!    ACM Transactions on Mathematical Software,
!    Volume 12, Number 4, December 1986, pages 362-376.
!
!    Pierre L'Ecuyer,
!    Random Number Generation,
!    in Handbook of Simulation,
!    edited by Jerry Banks,
!    Wiley, 1998,
!    ISBN: 0471134031,
!    LC: T57.62.H37.
!
!    Peter Lewis, Allen Goodman, James Miller,
!    A Pseudo-Random Number Generator for the System/360,
!    IBM Systems Journal,
!    Volume 8, Number 2, 1969, pages 136-143.
!
!  Parameters:
!
!    Input/output, integer ( kind = 4 ) SEED, the "seed" value, which
!    should NOT be 0.  On output, SEED has been updated.
!
!    Output, real ( kind = 4 ) R4_UNIFORM_01, a new pseudorandom variate,
!    strictly between 0 and 1.
!

function r4_uniform_01 ( seed )

  implicit none

  integer ( kind = 4 ), parameter :: i4_huge = 2147483647
  integer ( kind = 4 ) k
  integer ( kind = 4 ) seed
  real ( kind = 4 ) r4_uniform_01

  if ( seed == 0 ) then
    write ( *, '(a)' ) ' '
    write ( *, '(a)' ) 'R4_UNIFORM_01 - Fatal error!'
    write ( *, '(a)' ) '  Input value of SEED = 0.'
    stop 1
  end if

  k = seed / 127773

  seed = 16807 * ( seed - k * 127773 ) - k * 2836

  if ( seed < 0 ) then
    seed = seed + i4_huge
  end if

  r4_uniform_01 = real ( seed, kind = 4 ) * 4.656612875E-10

  return
end

!PROGRAM test  
!
! IMPLICIT none
!
! INTEGER :: K,ISEED
! REAL    :: RVAL
! REAL    :: R4_NORMAL_01,R4_UNIFORM_01
! REAL    :: CNTR,SUMX,SUM2
!
! CNTR=0.0
! SUMX=0.0
! SUM2=0.0
!
! iseed=-320
!
! DO K=1,10000
!    RVAL=r4_normal_01(iseed)
!### RVAL=r4_uniform_01(iseed)
!
!    CNTR=CNTR+1.0
!    SUMX=SUMX+RVAL
!    SUM2=SUM2+RVAL*RVAL
!  END DO
!
!  WRITE(*,*) SUMX/CNTR, SQRT((CNTR*SUM2-SUMX*SUMX)/(CNTR*(CNTR-1.0)))
!
!END PROGRAM test 
