!------------------------------------------------------------------------------
! Computes the turbulent velocities for the particle by linear interpolation
! of the turbulent velocities at the particle position, and reducing those
! values by the autocorrelation, and adding it to the particles previous
! turbulent velocities. Note W'/Sigma-W is saved for a particle rather than W'
!------------------------------------------------------------------------------
! Last Revised: 30 Jul 2020 - added option to compute variable time scale
!               08 Aug 2020 - upated common block
!------------------------------------------------------------------------------

SUBROUTINE pardsp(k1,k2,tf,dt,np,nrand,delh,xpos,ypos,zpos, &
                  uprime,vprime,wprime,wratio)

  USE metval, ONLY: e,uu,vv,ww
  USE ranval

  IMPLICIT NONE

!-------------------------------------
! argument list variables 

  INTEGER,      INTENT(IN)     :: k1,k2       ! time index (previous and next)
  REAL,         INTENT(IN)     :: tf          ! time interpolation factor
  INTEGER,      INTENT(IN)     :: dt          ! advection time step (sec) 
  INTEGER,      INTENT(IN)     :: np          ! sequential particle number
  INTEGER,      INTENT(IN)     :: nrand       ! index pointer for random number
  REAL,         INTENT(IN)     :: delh        ! vertical cell spacing (m)  
  REAL,         INTENT(IN)     :: xpos        ! EW horizontal particle position
  REAL,         INTENT(IN)     :: ypos        ! NS horizontal particle position
  REAL,         INTENT(IN)     :: zpos        ! vertical position (index)
  REAL,         INTENT(INOUT)  :: uprime      ! u' turbulent velocity (x)
  REAL,         INTENT(INOUT)  :: vprime      ! v' turbulent velocity (y)
  REAL,         INTENT(INOUT)  :: wprime      ! w' turbulent velocity (z)
  REAL,         INTENT(INOUT)  :: wratio      ! w'/sigw ratio

!--------------------------------------
! internally defined variables

  REAL          :: tratiov,tratioh   ! ratio for time step stability
  REAL          :: tke1,tke2         ! turbulent kinetic energy 
  REAL          :: tkemax            ! maximum TKE
  REAL          :: tkemin,pblmin     ! minimum values            
  REAL          :: vscale            ! vertical turublent time scale
  REAL          :: uvar, vvar        ! velocity sqrt(variances) at x,y   
  REAL          :: uvar1,vvar1,wvar1 ! time interpolated sqrt(variances)
  REAL          :: uvar2,vvar2,wvar2 ! time interpolated sqrt(variances)
  REAL          :: urand,vrand,wrand ! random velocities when R=1
  REAL          :: sigwt,sigwb,sigdz ! profile of velocity variances
  REAL          :: sigw              ! velocity sigma sqrt(wprime)
  REAL          :: rauto             ! velocity autocorrelation
  REAL          :: pauto             ! product of velocity autocorrelation
  REAL          :: delz              ! vertical offset to compute gradient
  
  INTEGER       :: nxp,nyp,nzp       ! grid limits
  INTEGER       :: kzmax             ! model top (m)
  INTEGER       :: cgratio           ! number of conc cells to WRF cells
  INTEGER       :: nb                ! bin number for turblent time scale

!-------------------------------------
  INTERFACE
  SUBROUTINE ADV3NT(S,XT,YT,ZT,SS)
  IMPLICIT NONE
  REAL,      INTENT(IN)    :: s(:,:,:)      ! field for interpolation
  REAL,      INTENT(IN)    :: xt,yt,zt      ! position of interpolated value
  REAL,      INTENT(OUT)   :: ss            ! value of S at x1,y1,z1
  END SUBROUTINE adv3nt
  END INTERFACE
!-------------------------------------

  COMMON /stblty/ tratiov,tratioh,tkemin,tkemax,pblmin
  COMMON /gridpt/ nxp,nyp,nzp,kzmax,cgratio

!------------------------------------
! horizontal

! turbulent sqrt(variance) at particle position
! interpolated to the current time
  CALL ADV3NT(UU(:,:,:,K1),XPOS,YPOS,ZPOS,UVAR1)
  CALL ADV3NT(UU(:,:,:,K2),XPOS,YPOS,ZPOS,UVAR2)
  UVAR=(UVAR2-UVAR1)*TF+UVAR1

  CALL ADV3NT(VV(:,:,:,K1),XPOS,YPOS,ZPOS,VVAR1)
  CALL ADV3NT(VV(:,:,:,K2),XPOS,YPOS,ZPOS,VVAR2)
  VVAR=(VVAR2-VVAR1)*TF+VVAR1

! random contribution to the new particle turbulence value 
  URAND=rannum(nrand)*uvar
  VRAND=rannum(nrand+1)*vvar

! determine vertical turbulence bin from particle number
  NB=MOD(NP,nbinh)+1

! autocorrelation function based upon time step and TL
! see main program where RAUTO=EXP(-ABS(DT)/HSCALE)
  RAUTO=RAUTOH(NB,DT)
! see main program where PAUTO=SQRT(1.0-RAUTO*RAUTO)
  PAUTO=PAUTOH(NB,DT)

! compute new turbulent velocity component
  UPRIME=RAUTO*UPRIME+PAUTO*URAND
  VPRIME=RAUTO*VPRIME+PAUTO*VRAND

!-------------------------------------
! vertical

! sqrt(variance) at particle position
  CALL ADV3NT(WW(:,:,:,K1),XPOS,YPOS,ZPOS,WVAR1)
  CALL ADV3NT(WW(:,:,:,K2),XPOS,YPOS,ZPOS,WVAR2)
  SIGW=(WVAR2-WVAR1)*TF+WVAR1

  IF(SIGW.LE.0.0)THEN
!    this should never happen because turbulence has a non-zero
!    minimum value 
     WRATIO=0.0
     WPRIME=0.0
     RETURN
  END IF

! compute new turbulent velocity
  WRAND=rannum(nrand+2)*SIGW

! check for position near top of domain for gradient calculation
! and adjust down by one grid point if needed
  DELZ=0.0
  IF(INT(ZPOS).EQ.NZP) DELZ=-1.0

! vertical gradient of the velocity variance
  CALL ADV3NT(WW(:,:,:,K1),XPOS,YPOS,(ZPOS+DELZ+1.0),WVAR1)
  CALL ADV3NT(WW(:,:,:,K2),XPOS,YPOS,(ZPOS+DELZ+1.0),WVAR2)
  SIGWT=(WVAR2-WVAR1)*TF+WVAR1

  CALL ADV3NT(WW(:,:,:,K1),XPOS,YPOS,(ZPOS+DELZ),WVAR1)
  CALL ADV3NT(WW(:,:,:,K2),XPOS,YPOS,(ZPOS+DELZ),WVAR2)
  SIGWB=(WVAR2-WVAR1)*TF+WVAR1
  SIGDZ=(SIGWT-SIGWB) / DELH

! compute new turbulent velocity (wprime) and new ratio for autocorrelation
! correction for non-homogeneous turbulence: Chock and Winkler (1994)
! J. Geophys. Res., Vol 99, No D1, Pages 1033-1041 and also
! Wilson, J.D., B.J. Legg, and D.J. Thomson, 1983. Calculation of particle 
! trajectories in the presence of a gradient in turbulent velocity variance, 
! B. Layer Meteorol., 27, 163-169

! determine vertical time scale (TKE at minimum suggests stable!)
  CALL ADV3NT(E(:,:,:,K1),XPOS,YPOS,ZPOS,TKE1)
  CALL ADV3NT(E(:,:,:,K2),XPOS,YPOS,ZPOS,TKE2)

! determine vertical turbulence bin from particle number
  NB=MOD(NP,nbinv)+1

! main defines RAUTO=EXP(-DT/VSCALE)) and PAUTO=SQRT(1-RAUTO**2)
  IF(TKE1.LE.TKEMIN.AND.TKE2.LE.TKEMIN)THEN
     VSCALE=VSCALES(NB)
     RAUTO=RAUTOS(NB,DT)
     PAUTO=PAUTOS(NB,DT)
  ELSE
     VSCALE=VSCALEU(NB)
     RAUTO=RAUTOU(NB,DT)
     PAUTO=PAUTOU(NB,DT)
  END IF

! wratio is conserved from particle, not wprime
  WPRIME=WRATIO*SIGW

! recompute wratio and wprime (used for turbulent displacement)
  WRATIO=RAUTO*WRATIO+PAUTO*WRAND/SIGW+VSCALE*(1.0-RAUTO)*SIGDZ
  WPRIME=WRATIO*(SIGW+(WPRIME*DT*SIGDZ))

END SUBROUTINE pardsp
