! ----------------------------------------------------------------------------
! Outputs the concentrations as a SVG graphic using color filled rectangles 
! A new file is created for each new output time period.  
! ----------------------------------------------------------------------------
! Last Revised: 29 Jan 2021 - initial version
! ----------------------------------------------------------------------------

SUBROUTINE SVGOUT(junit,kunit,svgfile,mapfile,datebgn,dateend,  &
                  thin,x0,y0,gbase,cfactor,cs)

  USE metval, ONLY: zt,yd,xd,glat,glon

  IMPLICIT NONE

  INTEGER, INTENT(IN)       :: junit,kunit
  CHARACTER(256),INTENT(IN) :: svgfile
  CHARACTER(256),INTENT(IN) :: mapfile
  INTEGER, INTENT(IN)       :: datebgn(6)
  INTEGER, INTENT(IN)       :: dateend(6) 
  LOGICAL, INTENT(IN)       :: thin
  REAL,    INTENT(IN)       :: x0,y0     
  REAL,    INTENT(IN)       :: gbase(9)  
  REAL,    INTENT(IN)       :: cfactor
  REAL,    INTENT(IN)       :: cs(:,:,:)   

! defines display window in pixel units
  REAL               :: wx,wy          ! width of concentration cell
  INTEGER, PARAMETER :: pxmin = 10
  INTEGER, PARAMETER :: pymin = 10
  INTEGER, PARAMETER :: pxmax = 600
  INTEGER, PARAMETER :: pymax = 600

  REAL, ALLOCATABLE :: cval(:,:)       ! concentrations for display 

  INTEGER      :: I,J,K,II,JJ,KK  
  INTEGER      :: KTOP  
  INTEGER      :: KFILE = 0            ! output file counter
  CHARACTER(80):: FNAME

  REAL         :: CTOT
  REAL         :: CGSZXY
  REAL         :: X,Y,XP,YP            ! position on meteorology grid
  REAL         :: XDIS,YDIS            ! concentration grid spacing
  REAL         :: XLON,YLAT            ! concentration grid position 
  REAL         :: LAT1,LON1            ! lower left of display
  REAL         :: LAT2,LON2            ! upper right of display
  REAL         :: RADIUS               ! minimum display radius (km)
  REAL         :: XS,YS                ! source location concentration grid
  REAL         :: XYR                  ! radius in conc grid units

! concentration grid domain           
  INTEGER      :: ij    
  INTEGER      :: imin,imax,jmin,jmax  
  INTEGER      :: nxp,nyp,nzp,nxc,nyc 
  INTEGER      :: kzmax        ! model top (m)
  INTEGER      :: cgratio      ! number of concentration grid cells to WRF cells
  INTEGER      :: yy           ! two digit year

! concentration grid configuration
  CHARACTER(4) :: polid        ! pollutant identification
  INTEGER      :: conlvl       ! height m-AGL of grid plane
  REAL         :: clat         ! lower left corner concengration grid
  REAL         :: clon         ! lower left corner concentration grid
  REAL         :: dlat         ! grid spacing in degrees latitude
  REAL         :: dlon         ! grid spacing in degrees longitude
  INTEGER      :: nlon         ! number of longitude grid points
  INTEGER      :: nlat         ! number of latitude grid points

  LOGICAL      :: ftest

! whole number contour values and associated R,G,B
  integer*4 nc
  parameter (nc = 10)
  real*4 c(nc)
  integer*4 r(nc),g(nc),b(nc)
  data c /20.,50.,100.,200.,500.,1000.,2000.,5000.,10000.,20000./
  data r /225,200,178,  110,   0,    0,    0,    0,   255,   255/
  data g /210,160,140,  150, 255,  180,    0,  255,   255,     0/
  data b /180,120, 80,  115, 255,  255,  255,    0,     0,     0/

  COMMON /gridpt/ nxp,nyp,nzp,kzmax,cgratio
  COMMON /conval/ polid,conlvl,clat,clon,dlat,dlon,nlat,nlon

  SAVE cval,kfile

!---------------------------------------------------
  INTERFACE
  SUBROUTINE MAPBDY(junit,kunit,imin,jmin,imax,jmax, &
                    pxmin,pymin,pxmax,pymax,cgratio,gbase)
  IMPLICIT none
  INTEGER, INTENT(IN) :: junit,kunit
  INTEGER, INTENT(IN) :: imin,jmin,imax,jmax
  INTEGER, INTENT(IN) :: pxmin,pymin,pxmax,pymax
  INTEGER, INTENT(IN) :: cgratio
  REAL,    INTENT(IN) :: gbase(9)
  END SUBROUTINE mapbdy
  END INTERFACE
!---------------------------------------------------

  radius=100.0
  
! find the top of the output layer 
  KK=2
  DO WHILE (KK.LE.NZP.AND.ZT(KK).LE.CONLVL)
     KK=KK+1      
  END DO
  KTOP=KK-1

! concentration grid limits
  nxc=1+cgratio*(nxp-1)
  nyc=1+cgratio*(nyp-1)
  IF(.NOT.ALLOCATED(cval)) ALLOCATE (cval(nxc,nyc))

! grid needs to cover the source location
  xs=1.0+cgratio*(x0-1.0)
  ys=1.0+cgratio*(y0-1.0)

! add minimum radius in grid units
  xyr=1000.0*cgratio*radius/XD(NINT(x0),NINT(y0))

! domain in concentration grid units
  imax=NINT(xs+1.1*xyr)
  jmax=NINT(ys+1.1*xyr)
  imin=NINT(xs-1.1*xyr)
  jmin=NINT(ys-1.1*xyr) 

!-----------------------------------------------------
! copy concentration data into single layer display grid

  DO II=1,NXC
  DO JJ=1,NYC

     CVAL(II,JJ)=0.0
     CTOT=SUM(CS(II,JJ,2:KTOP))*CFACTOR

!    process only non-zero grid points
     IF(CTOT.GT.0.0)THEN

        IF(cgratio.EQ.1)THEN
!          use pre-computed meteorology grid conversions
           YLAT=GLAT(II,JJ)
           XLON=GLON(II,JJ)
           XDIS=XD(II,JJ)
           YDIS=YD(II,JJ)

        ELSE
!          convert concentration grid point to meteorology grid
           XP=1.0+(II-1.0)/CGRATIO
           YP=1.0+(JJ-1.0)/CGRATIO

!          recompute position and resolution
           CALL CXY2LL(GBASE,XP,YP,YLAT,XLON)
!          meters per concentration grid cell
           XDIS=CGSZXY(GBASE,XP,YP)*1000.0
           YDIS=XDIS
        END IF

        CVAL(II,JJ)=CTOT*CGRATIO*CGRATIO/(XDIS*YDIS*ZT(KTOP))
        IMAX=MAX(II,IMAX)
        JMAX=MAX(JJ,JMAX)
        IMIN=MIN(II,IMIN)
        JMIN=MIN(JJ,JMIN)

     END IF
  END DO
  END DO

!-----------------------------------------------------
! lat-lon limits of concentration display

  XP=1.0+(IMIN-1.0)/CGRATIO
  YP=1.0+(JMIN-1.0)/CGRATIO
  CALL CXY2LL(GBASE,XP,YP,LAT1,LON1)

  XP=1.0+(IMAX-1.0)/CGRATIO
  YP=1.0+(JMAX-1.0)/CGRATIO
  CALL CXY2LL(GBASE,XP,YP,LAT2,LON2)
 
!-----------------------------------------------------
! color fill concentration grid

  KFILE=KFILE+1
  K=INDEX(svgfile,' ')-1
  WRITE(FNAME,'(A,I2.2,A)')SVGFILE(1:K),KFILE,'.html'
  OPEN(UNIT=JUNIT,FILE=FNAME)

  IF(.NOT.thin)THEN
!    add header information to output file 
!    note that a file without can be easily converted with ImageMagick
     WRITE(JUNIT,'(A)')'<!DOCTYPE html>'
     WRITE(JUNIT,'(A)')'<head>'
     WRITE(JUNIT,'(A)')'<meta charset="utf-8">'
     WRITE(JUNIT,'(2A)')'<meta name="Author" CONTENT="Roland Draxler,',  &
                        ' http://www.meteozone.com">'
     WRITE(JUNIT,'(A,2(F6.2,F7.2),A)')    &
              '<meta name="Description" CONTENT="',lat1,lon1,lat2,lon2,'">'
     WRITE(JUNIT,'(A)')'</head>'
     WRITE(JUNIT,'(A)')'<body>'
  END IF

  WRITE(JUNIT,'(A,4I4,A)') '<svg height="800" width="800" viewBox="', &
                            pxmin,pymin,pxmax,pymax,'" >'

! display domain
  WRITE(JUNIT,'(4(A,I4),A)')      &
     '<rect x="',pxmin,           &
         '" y="',pymin,           &
     '" width="',pxmax,           &
    '" height="',pymax,           &
      '" fill="none" stroke="black" stroke-width="2" />' 

! number of display pixels per grid cell
  ij=MAX(imax-imin,jmax-jmin)
  wx=(pxmax-pxmin)/float(ij)
  wy=(pymax-pymin)/float(ij)

!-----------------------------------------------------
! fill in concentration grid

  DO j=jmin,jmax
  DO i=imin,imax

!    find contour value color
     k=0
     DO kk=1,nc
        IF(c(kk).LT.cval(i,j))k=kk
     END DO

     IF(k.GT.0)THEN
!       coordinates suitable graphics display (x:lft->rgt y:top->bot)
        y = pymax - (j-jmin)*wx 
        x = (i-imin)*wy + pxmin
        WRITE(JUNIT,'(A,F6.1,A,F6.1,A,F4.1,A,F4.1,A,3(I3,A))')    &
           '<rect x="', X,       &
               '" y="', Y,       &
           '" width="',(wx+1.0), &
          '" height="',wy,       &
            '" fill="rgb(',r(k),",",g(k),",",b(k),')" />' 
     END IF

  END DO
  END DO

!-----------------------------------------------------
! map overlay

  INQUIRE(FILE=TRIM(mapfile),EXIST=FTEST)
  IF(FTEST)THEN
     OPEN(KUNIT,FILE=TRIM(mapfile),STATUS='OLD')
     CALL MAPBDY(junit,kunit,imin,jmin,imax,jmax, &
                 pxmin,pymin,pxmax,pymax,cgratio,gbase)
     CLOSE(KUNIT)
  ELSE
     WRITE(*,*)'Map background not found: ',TRIM(mapfile)
  END IF

!-----------------------------------------------------
! draw source and minimum display radius

  x = (xs-imin)*wy + pxmin
  y = pymax - (ys-jmin)*wx 
  WRITE(JUNIT,'(A,F6.1,A,F6.1,A,F6.1,A)')  &
    '<circle cx="',X,                      &
          '" cy="',Y,                      &
           '" r="',(XYR*WX),               &
      '" fill="none" stroke="black" stroke-width="0.5" />' 

  WRITE(JUNIT,'(A,F6.1,A,F6.1,A,F6.1,A)')  &
    '<circle cx="',X,                      &
          '" cy="',Y,                      &
           '" r="',2.5,                    &
      '" fill="black" stroke-width="0" />' 

!-----------------------------------------------------
! add index for contour colors

  j=pymax-5
  DO k=1,nc
     i=pxmin+(k*50)-35   
     WRITE(JUNIT,'(2(A,I3),2A,3(I3,A))')     &
         '<rect x="', I,                     &
             '" y="', J,                     &
         '" width="15" height="10"',         &
           ' fill="rgb(',r(k),",",g(k),",",b(k),')" />' 

     WRITE(JUNIT,'(2(A,I3.3),A,I5,A)')       &
         '<text x="', I,                     &
             '" y="',(J-5),                  &
      '" stroke="black" stroke-width="0.5">',& 
              INT(c(k)),'</text>'          
  END DO

! concentration scaling factor
     WRITE(JUNIT,'(2(A,I3.3),A,1P,E7.1,A)')     &
         '<text x="',(pxmax-70),             &
             '" y="',(pymax-2),              &
      '" stroke="black" stroke-width="1">',  & 
              (1.0/cfactor),'</text>'          

!-----------------------------------------------------
! start and stop sampling times

! write record for sampling begin time
  yy=datebgn(1)-(datebgn(1)/100)*100
  WRITE(JUNIT,'(2(A,I3.3),A,5I2.2,A)')    &
      '<text x="',(pxmin+20),             &
          '" y="',(pymin+15),             &
   '" stroke="black" stroke-width="0.5">',& 
              yy,datebgn(2:5),'</text>'          

! write record for sampling end time
  yy=dateend(1)-(dateend(1)/100)*100
  WRITE(JUNIT,'(2(A,I3.3),A,5I2.2,A)')    &
      '<text x="',(pxmax-90),             &
          '" y="',(pymin+15),             &
   '" stroke="black" stroke-width="0.5">',& 
              yy,dateend(2:5),'</text>'          

  WRITE(JUNIT,'(A)')'</svg>'
  IF(.NOT.thin)THEN
     WRITE(JUNIT,'(A)')'</body>'
     WRITE(JUNIT,'(A)')'</html>'
  END IF
  CLOSE(JUNIT)

END SUBROUTINE svgout  
